@@ -21,7 +21,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QValidator>
 
+class QString;
+class QObject;
+
 class HexStringValidator : public QValidator {
+	Q_OBJECT
 public:
 	HexStringValidator(QObject * parent);
 
