@@ -20,21 +20,30 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define IBINARY_20070718_H_
 
 #include "API.h"
-#include "IRegion.h"
 #include "Types.h"
+#include <QVector>
 #include <memory>
 
+class IRegion;
+
 class EDB_EXPORT IBinary {
 public:
-	virtual ~IBinary() {}
+	struct Header {
+		edb::address_t address;
+		size_t         size;
+		// TODO(eteran): maybe label/type/etc...
+	};
+public:
+	virtual ~IBinary() = default;
 
 public:
 	virtual bool native() const = 0;
-	virtual bool validate_header() = 0;
 	virtual edb::address_t calculate_main() = 0;
 	virtual edb::address_t entry_point() = 0;
 	virtual size_t header_size() const = 0;
 	virtual const void *header() const = 0;
+	virtual QVector<Header> headers() const = 0;
+	virtual edb::address_t base_address() const = 0;
 
 public:
 	// optional, and platform specific:
@@ -43,7 +52,7 @@ public:
 	virtual edb::address_t debug_pointer() { return 0; }
 
 public:
-	typedef std::unique_ptr<IBinary> (*create_func_ptr_t)(const IRegion::pointer &);
+	typedef std::unique_ptr<IBinary> (*create_func_ptr_t)(const std::shared_ptr<IRegion> &);
 };
 
 #endif
