@@ -20,14 +20,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define IDEBUG_EVENT_HANDLER_20061101_H_
 
 #include "Types.h"
-#include "IDebugEvent.h"
+#include <memory>
+
+class IDebugEvent;
 
 class IDebugEventHandler {
 public:
-	virtual ~IDebugEventHandler() {}
+	virtual ~IDebugEventHandler() = default;
 
 public:
-	virtual edb::EVENT_STATUS handle_event(const IDebugEvent::const_pointer &event) = 0;
+	virtual edb::EVENT_STATUS handle_event(const std::shared_ptr<IDebugEvent> &event) = 0;
 };
 
 #endif
