@@ -20,16 +20,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define ITHREAD_20150529_H_
 
 #include "Types.h"
-#include <memory>
+#include "OSTypes.h"
+#include "Status.h"
 
 class State;
 
 class IThread {
 public:
-	typedef std::shared_ptr<IThread> pointer;
-public:
-	virtual ~IThread() {}
-	
+	virtual ~IThread() = default;
+
 public:
 	virtual edb::tid_t tid() const = 0;
 	virtual QString name() const = 0;
@@ -37,22 +36,19 @@ public:
 	virtual edb::address_t instruction_pointer() const = 0;
 	virtual QString runState() const = 0;
 
-public:
-	virtual void get_state(State *state) = 0;	
-	virtual void set_state(const State &state) = 0;
-
-public:
-	virtual void step() = 0;
-	virtual void step(edb::EVENT_STATUS status) = 0;	
-	virtual void resume() = 0;
-	virtual void resume(edb::EVENT_STATUS status) = 0;
-	virtual void stop() = 0;
-
-#if 0	
 public:
 	virtual void get_state(State *state) = 0;
 	virtual void set_state(const State &state) = 0;
-#endif
+
+public:
+	virtual Status step() = 0;
+	virtual Status step(edb::EVENT_STATUS status) = 0;
+	virtual Status resume() = 0;
+	virtual Status resume(edb::EVENT_STATUS status) = 0;
+	virtual Status stop() = 0;
+
+public:
+	virtual bool isPaused() const = 0;
 };
 
 #endif
