@@ -23,13 +23,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <cstdint>
 
 class QLongValidator : public QValidator {
+	Q_OBJECT
 public:
 	typedef std::int64_t value_type;
 
 public:
-	explicit QLongValidator(QObject *parent = 0);
-	QLongValidator(value_type minimum, value_type maximum, QObject *parent = 0);
-	virtual ~QLongValidator() {}
+    explicit QLongValidator(QObject *parent = nullptr);
+    QLongValidator(value_type minimum, value_type maximum, QObject *parent = nullptr);
+	virtual ~QLongValidator() = default;
 
 public:
 	value_type bottom() const;
