@@ -23,13 +23,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <cstdint>
 
 class QULongValidator : public QValidator {
+	Q_OBJECT
 public:
 	typedef std::uint64_t value_type;
 
 public:
-	explicit QULongValidator(QObject *parent = 0);
-	QULongValidator(value_type minimum, value_type maximum, QObject *parent = 0);
-	virtual ~QULongValidator() {}
+    explicit QULongValidator(QObject *parent = nullptr);
+    QULongValidator(value_type minimum, value_type maximum, QObject *parent = nullptr);
+	virtual ~QULongValidator() = default;
 
 public:
 	value_type bottom() const;
