@@ -22,12 +22,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "API.h"
 #include "Types.h"
 #include <QString>
-#include <memory>
 
 class EDB_EXPORT Symbol {
-public:
-	typedef std::shared_ptr<Symbol> pointer;
-
 public:
 	QString        file;
 	QString        name;
