@@ -19,28 +19,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef ARCHTYPES_20071127_H_
 #define ARCHTYPES_20071127_H_
 
-#include <cstdint>
-#include <QtGlobal>
 #include "Instruction.h"
-
-#if INTPTR_MAX==INT32_MAX
-
-#define EDB_X86
-static constexpr const bool EDB_IS_64_BIT=false;
-static constexpr const bool EDB_IS_32_BIT=true;
-
-#elif INTPTR_MAX==INT64_MAX
-
-#define EDB_X86_64
-static constexpr const bool EDB_IS_64_BIT=true;
-static constexpr const bool EDB_IS_32_BIT=false;
-
-#endif
+#include "Types.h"
 
 namespace edb {
-	typedef value16                                    seg_reg_t;
-	typedef CapstoneEDB::Instruction                   Instruction;
-	typedef Instruction::operand_type                  Operand;
+
+typedef value16                   seg_reg_t;
+typedef CapstoneEDB::Instruction  Instruction;
+typedef CapstoneEDB::Operand      Operand;
+
 }
 
 #endif
