@@ -22,9 +22,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/types.h> // for pid_t/uid_t
 
 namespace edb {
-	using   ::pid_t;
-	using   ::uid_t;
-	typedef ::pid_t tid_t;
+	using ::pid_t;
+	using ::uid_t;
+	using tid_t = ::pid_t;
 }
 
 #endif
