@@ -17,19 +17,27 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "Analyzer.h"
-#include "OptionsPage.h"
 #include "AnalyzerWidget.h"
-#include "SpecifiedFunctions.h"
+#include "Configuration.h"
+#include "DialogXRefs.h"
+#include "Function.h"
 #include "IBinary.h"
 #include "IDebugger.h"
+#include "IProcess.h"
 #include "ISymbolManager.h"
+#include "IThread.h"
 #include "Instruction.h"
 #include "MemoryRegions.h"
+#include "OptionsPage.h"
+#include "Prototype.h"
+#include "SpecifiedFunctions.h"
 #include "State.h"
 #include "Util.h"
 #include "edb.h"
 
 #include <QCoreApplication>
+#include <QDir>
+#include <QFileInfo>
 #include <QHash>
 #include <QMainWindow>
 #include <QMenu>
@@ -57,7 +65,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #endif
 
-namespace Analyzer {
+namespace AnalyzerPlugin {
 
 namespace {
 
@@ -67,7 +75,7 @@ const int MIN_REFCOUNT = 2;
 // Name: module_entry_point
 // Desc:
 //------------------------------------------------------------------------------
-edb::address_t module_entry_point(const IRegion::pointer &region) {
+edb::address_t module_entry_point(const std::shared_ptr<IRegion> &region) {
 
 	edb::address_t entry = 0;
 	if(auto binary_info = edb::v1::get_binary_info(region)) {
@@ -105,11 +113,11 @@ QMenu *Analyzer::menu(QWidget *parent) {
 	if(!menu_) {
 		menu_ = new QMenu(tr("Analyzer"), parent);
 		menu_->addAction(tr("Show &Specified Functions"), this, SLOT(show_specified()));
-				
+
 		if(edb::v1::debugger_core) {
 			menu_->addAction(tr("&Analyze %1's Region").arg(edb::v1::debugger_core->instruction_pointer().toUpper()), this, SLOT(do_ip_analysis()), QKeySequence(tr("Ctrl+A")));
 		}
-		
+
 		menu_->addAction(tr("&Analyze Viewed Region"), this, SLOT(do_view_analysis()), QKeySequence(tr("Ctrl+Shift+A")));
 
 		// if we are dealing with a main window (and we are...)
@@ -158,12 +166,12 @@ void Analyzer::show_specified() {
 //------------------------------------------------------------------------------
 void Analyzer::do_ip_analysis() {
 	if(IProcess *process = edb::v1::debugger_core->process()) {
-		if(IThread::pointer thread = process->current_thread()) {
+		if(std::shared_ptr<IThread> thread = process->current_thread()) {
 			State state;
 			thread->get_state(&state);
-		
+
 			const edb::address_t address = state.instruction_pointer();
-			if(IRegion::pointer region = edb::v1::memory_regions().find_region(address)) {
+			if(std::shared_ptr<IRegion> region = edb::v1::memory_regions().find_region(address)) {
 				do_analysis(region);
 			}
 		}
@@ -185,13 +193,39 @@ void Analyzer::do_view_analysis() {
 void Analyzer::mark_function_start() {
 
 	const edb::address_t address = edb::v1::cpu_selected_address();
-	if(IRegion::pointer region = edb::v1::memory_regions().find_region(address)) {
-		qDebug("Added %s to the list of known functions",qPrintable(address.toPointerString()));
+	if(std::shared_ptr<IRegion> region = edb::v1::memory_regions().find_region(address)) {
+		qDebug("Added %s to the list of known functions", qPrintable(address.toPointerString()));
 		specified_functions_.insert(address);
 		invalidate_dynamic_analysis(region);
 	}
 }
 
+//------------------------------------------------------------------------------
+// Name: mark_function_start
+// Desc:
+//------------------------------------------------------------------------------
+void Analyzer::show_xrefs() {
+
+	const edb::address_t address = edb::v1::cpu_selected_address();
+	
+	auto dialog = new DialogXRefs(edb::v1::debugger_ui);
+	
+	for(const RegionData &data : analysis_info_) {
+		for(const BasicBlock &bb : data.basic_blocks) {	
+			QVector<QPair<edb::address_t, edb::address_t>> refs = bb.refs();
+			for(auto it = refs.begin(); it != refs.end(); ++it) {
+			
+				if(it->second == address) {					
+					dialog->addReference(*it);
+				}	
+			}
+		}
+	}
+	
+	dialog->setWindowTitle(tr("X-Refs For %1").arg(address.toPointerString()));
+	dialog->show();
+}
+
 //------------------------------------------------------------------------------
 // Name: goto_function_start
 // Desc:
@@ -244,12 +278,15 @@ QList<QAction *> Analyzer::cpu_context_menu() {
 	auto action_goto_function_start = new QAction(tr("Goto Function Start"), this);
 	auto action_goto_function_end   = new QAction(tr("Goto Function End"), this);
 	auto action_mark_function_start = new QAction(tr("Mark As Function Start"), this);
+	auto action_xrefs               = new QAction(tr("Show X-Refs"), this);
 
-	connect(action_find, SIGNAL(triggered()), this, SLOT(do_view_analysis()));
+	connect(action_find, SIGNAL(triggered()),                this, SLOT(do_view_analysis()));
 	connect(action_goto_function_start, SIGNAL(triggered()), this, SLOT(goto_function_start()));
 	connect(action_goto_function_end, SIGNAL(triggered()),   this, SLOT(goto_function_end()));
 	connect(action_mark_function_start, SIGNAL(triggered()), this, SLOT(mark_function_start()));
-	ret << action_find << action_goto_function_start << action_goto_function_end << action_mark_function_start;
+	connect(action_xrefs, SIGNAL(triggered()),               this, SLOT(show_xrefs()));
+	
+	ret << action_find << action_goto_function_start << action_goto_function_end << action_mark_function_start << action_xrefs;
 
 	return ret;
 }
@@ -258,7 +295,7 @@ QList<QAction *> Analyzer::cpu_context_menu() {
 // Name: do_analysis
 // Desc:
 //------------------------------------------------------------------------------
-void Analyzer::do_analysis(const IRegion::pointer &region) {
+void Analyzer::do_analysis(const std::shared_ptr<IRegion> &region) {
 	if(region && region->size() != 0) {
 		QProgressDialog progress(tr("Performing Analysis"), 0, 0, 100, edb::v1::debugger_ui);
 		connect(this, SIGNAL(update_progress(int)), &progress, SLOT(setValue(int)));
@@ -296,9 +333,9 @@ void Analyzer::bonus_symbols(RegionData *data) {
 	Q_ASSERT(data);
 
 	// give bonus if we have a symbol for the address
-	const QList<Symbol::pointer> symbols = edb::v1::symbol_manager().symbols();
+	const QList<std::shared_ptr<Symbol>> symbols = edb::v1::symbol_manager().symbols();
 
-	for(const Symbol::pointer &sym: symbols) {
+	for(const std::shared_ptr<Symbol> &sym: symbols) {
 		const edb::address_t addr = sym->address;
 
 		if(data->region->contains(addr) && sym->is_code()) {
@@ -316,7 +353,7 @@ void Analyzer::bonus_marked_functions(RegionData *data) {
 
 	Q_ASSERT(data);
 
-	for(const edb::address_t addr: specified_functions_) {
+	Q_FOREACH(const edb::address_t addr, specified_functions_) {
 		if(data->region->contains(addr)) {
 			qDebug("[Analyzer] adding user marked function: <%s>", qPrintable(addr.toPointerString()));
 			data->known_functions.insert(addr);
@@ -390,16 +427,16 @@ void Analyzer::collect_functions(Analyzer::RegionData *data) {
 
 	// results
 	QHash<edb::address_t, BasicBlock> basic_blocks;
-	QHash<edb::address_t, Function>   functions;
+	FunctionMap                       functions;
 
 	// push all known functions onto a stack
 	QStack<edb::address_t> known_functions;
-	for(const edb::address_t function: data->known_functions) {
+	Q_FOREACH(const edb::address_t function, data->known_functions) {
 		known_functions.push(function);
 	}
 
 	// push all fuzzy function too...
-	for(const edb::address_t function: data->fuzzy_functions) {
+	Q_FOREACH(const edb::address_t function, data->fuzzy_functions) {
 		known_functions.push(function);
 	}
 
@@ -412,7 +449,7 @@ void Analyzer::collect_functions(Analyzer::RegionData *data) {
 			QStack<edb::address_t> blocks;
 			blocks.push(function_address);
 
-			Function func(function_address);
+			Function func;
 
 			// process are basic blocks that are known
 			while(!blocks.empty()) {
@@ -430,34 +467,37 @@ void Analyzer::collect_functions(Analyzer::RegionData *data) {
 							break;
 						}
 
-						const edb::Instruction inst(buffer, buffer + buf_size, address);
-						if(!inst) {
+						auto inst = std::make_shared<edb::Instruction>(buffer, buffer + buf_size, address);
+						if(!inst->valid()) {
 							break;
 						}
 
-						block.push_back(std::make_shared<edb::Instruction>(inst));
+						block.push_back(inst);
 
-						if(is_call(inst)) {
+						if(is_call(*inst)) {
 
 							// note the destination and move on
 							// we special case some simple things.
 							// also this is an opportunity to find call tables.
-							const edb::Operand &op = inst.operands()[0];
-							if(op.general_type() == edb::Operand::TYPE_REL) {
-								const edb::address_t ea = op.relative_target();
+							const auto op = inst->operand(0);
+							if(is_immediate(op)) {
+								const edb::address_t ea = op->imm;
 
 								// skip over ones which are: "call <label>; label:"
-								if(ea != address + inst.size()) {
+								if(ea != address + inst->byte_size()) {
 									known_functions.push(ea);
-									
+
 									if(!will_return(ea)) {
 										break;
 									}
+									
+									block.addRef(address, ea);
+									
 								}
-							} else if(op.general_type() == edb::Operand::TYPE_EXPRESSION) {
+							} else if(is_expression(op)) {
 								// looks like: "call [...]", if it is of the form, call [C + REG]
 								// then it may be a jump table using REG as an offset
-							} else if(op.general_type() == edb::Operand::TYPE_REGISTER) {
+							} else if(is_register(op)) {
 								// looks like: "call <reg>", this is this may be a callback
 								// if we can use analysis to determine that it's a constant
 								// we can figure it out...
@@ -465,18 +505,18 @@ void Analyzer::collect_functions(Analyzer::RegionData *data) {
 								// to see if we can know what the target is
 							}
 
-							
-						} else if(is_unconditional_jump(inst)) {
 
-							Q_ASSERT(inst.operand_count() >= 1);
-							const edb::Operand &op = inst.operands()[0];
+						} else if(is_unconditional_jump(*inst)) {
+
+							Q_ASSERT(inst->operand_count() >= 1);
+							const auto op = inst->operand(0);
+
+							// TODO(eteran): we need some heuristic for detecting when this is
+							//               a call/ret -> jmp optimization
+							if(is_immediate(op)) {
+								const edb::address_t ea = op->imm;
 
-							// TODO: we need some heuristic for detecting when this is
-							//       a call/ret -> jmp optimization
-							if(op.general_type() == edb::Operand::TYPE_REL) {
-								const edb::address_t ea = op.relative_target();
 
-								
 								if(functions.contains(ea)) {
 									functions[ea].add_reference();
 								} else if((ea - function_address) > 0x2000u) {
@@ -484,23 +524,30 @@ void Analyzer::collect_functions(Analyzer::RegionData *data) {
 								} else {
 									blocks.push(ea);
 								}
+								
+								block.addRef(address, ea);
 							}
 							break;
-						} else if(is_conditional_jump(inst)) {
+						} else if(is_conditional_jump(*inst)) {
 
-							Q_ASSERT(inst.operand_count() == 1);
-							const edb::Operand &op = inst.operands()[0];
+							Q_ASSERT(inst->operand_count() == 1);
+							const auto op = inst->operand(0);
 
-							if(op.general_type() == edb::Operand::TYPE_REL) {
-								blocks.push(op.relative_target());
-								blocks.push(address + inst.size());
+							if(is_immediate(op)) {
+							
+								const edb::address_t ea = op->imm;
+							
+								blocks.push(ea);
+								blocks.push(address + inst->byte_size());
+								
+								block.addRef(address, ea);
 							}
 							break;
-						} else if(is_terminator(inst)) {
+						} else if(is_terminator(*inst)) {
 							break;
 						}
 
-						address += inst.size();
+						address += inst->byte_size();
 					}
 
 					if(!block.empty()) {
@@ -521,16 +568,17 @@ void Analyzer::collect_functions(Analyzer::RegionData *data) {
 		}
 	}
 
+#if 0
 	qDebug() << "----------Basic Blocks----------";
 	for(auto it = basic_blocks.begin(); it != basic_blocks.end(); ++it) {
 		qDebug("%s:", qPrintable(it.key().toPointerString()));
 
-		for(auto j = it.value().begin(); j != it.value().end(); ++j) {
-			const instruction_pointer &inst = *j;
+		for(auto &&inst : it.value()) {
 			qDebug("\t%s: %s", qPrintable(edb::address_t(inst->rva()).toPointerString()), edb::v1::formatter().to_string(*inst).c_str());
 		}
 	}
 	qDebug() << "----------Basic Blocks----------";
+#endif
 
 	qSwap(data->basic_blocks, basic_blocks);
 	qSwap(data->functions, functions);
@@ -549,33 +597,35 @@ void Analyzer::collect_fuzzy_functions(RegionData *data) {
 
 		QHash<edb::address_t, int> fuzzy_functions;
 
+		quint8 *const first = &data->memory[0];
+		quint8 *const last  = &first[data->memory.size()];
+		quint8 *p           = first;
+
 		// fuzzy_functions, known_functions
 		for(edb::address_t addr = data->region->start(); addr != data->region->end(); ++addr) {
+			const edb::Instruction inst(p, last, addr);
+			if(inst) {
+				if(is_call(inst)) {
 
-			quint8 buf[edb::Instruction::MAX_SIZE];
-			if(const int buf_size = edb::v1::get_instruction_bytes(addr, buf)) {
-				const edb::Instruction inst(buf, buf + buf_size, addr);
-				if(inst) {
-					if(is_call(inst)) {
+					// note the destination and move on
+					// we special case some simple things.
+					// also this is an opportunity to find call tables.
+					const auto op = inst[0];
+					if(is_immediate(op)) {
+						const edb::address_t ea = op->imm;
 
-						// note the destination and move on
-						// we special case some simple things.
-						// also this is an opportunity to find call tables.
-						const edb::Operand &op = inst.operands()[0];
-						if(op.general_type() == edb::Operand::TYPE_REL) {
-							const edb::address_t ea = op.relative_target();
+						// skip over ones which are: "call <label>; label:"
+						if(ea != addr + inst.byte_size()) {
 
-							// skip over ones which are: "call <label>; label:"
-							if(ea != addr + inst.size()) {
-
-								if(!data->known_functions.contains(ea)) {
-									fuzzy_functions[ea]++;
-								}
+							if(!data->known_functions.contains(ea)) {
+								fuzzy_functions[ea]++;
 							}
 						}
 					}
 				}
 			}
+
+			++p;
 		}
 
 		// transfer results to data->fuzzy_functions
@@ -591,16 +641,23 @@ void Analyzer::collect_fuzzy_functions(RegionData *data) {
 // Name: analyze
 // Desc:
 //------------------------------------------------------------------------------
-void Analyzer::analyze(const IRegion::pointer &region) {
+void Analyzer::analyze(const std::shared_ptr<IRegion> &region) {
 
 	QTime t;
 	t.start();
 
 	RegionData &region_data = analysis_info_[region->start()];
+	qDebug() << "[Analyzer] Region name:" << region->name();
 
 	QSettings settings;
-	const bool fuzzy          = settings.value("Analyzer/fuzzy_logic_functions.enabled", true).toBool();
-	const QByteArray md5      = md5_region(region);
+	const bool fuzzy = settings.value("Analyzer/fuzzy_logic_functions.enabled", true).toBool();
+
+	const edb::address_t page_size = edb::v1::debugger_core->page_size();
+	const size_t page_count        = region->size() / page_size;
+
+	QVector<quint8> memory = edb::v1::read_pages(region->start(), page_count);
+
+	const QByteArray md5      = (!memory.isEmpty()) ? edb::v1::get_md5(memory) : QByteArray();
 	const QByteArray prev_md5 = region_data.md5;
 
 	if(md5 != prev_md5 || fuzzy != region_data.fuzzy) {
@@ -610,6 +667,7 @@ void Analyzer::analyze(const IRegion::pointer &region) {
 		region_data.fuzzy_functions.clear();
 		region_data.known_functions.clear();
 
+		region_data.memory = memory;
 		region_data.region = region;
 		region_data.md5    = md5;
 		region_data.fuzzy  = fuzzy;
@@ -618,13 +676,13 @@ void Analyzer::analyze(const IRegion::pointer &region) {
 			const char             *message;
 			std::function<void()> function;
 		} analysis_steps[] = {
-			{ "identifying executable headers...",                       std::bind(&Analyzer::ident_header,            this, &region_data) },
-			{ "adding entry points to the list...",                      std::bind(&Analyzer::bonus_entry_point,       this, &region_data) },
-			{ "attempting to add 'main' to the list...",                 std::bind(&Analyzer::bonus_main,              this, &region_data) },
-			{ "attempting to add functions with symbols to the list...", std::bind(&Analyzer::bonus_symbols,           this, &region_data) },
-			{ "attempting to add marked functions to the list...",       std::bind(&Analyzer::bonus_marked_functions,  this, &region_data) },
-			{ "attempting to collect functions with fuzzy analysis...",  std::bind(&Analyzer::collect_fuzzy_functions, this, &region_data) },
-			{ "collecting basic blocks...",                              std::bind(&Analyzer::collect_functions,       this, &region_data) },
+			{ "identifying executable headers...",                       [this, &region_data]() { ident_header(&region_data);            } },
+			{ "adding entry points to the list...",                      [this, &region_data]() { bonus_entry_point(&region_data);       } },
+			{ "attempting to add 'main' to the list...",                 [this, &region_data]() { bonus_main(&region_data);              } },
+			{ "attempting to add functions with symbols to the list...", [this, &region_data]() { bonus_symbols(&region_data);           } },
+			{ "attempting to add marked functions to the list...",       [this, &region_data]() { bonus_marked_functions(&region_data);  } },
+			{ "attempting to collect functions with fuzzy analysis...",  [this, &region_data]() { collect_fuzzy_functions(&region_data); } },
+			{ "collecting basic blocks...",                              [this, &region_data]() { collect_functions(&region_data);       } },
 		};
 
 		const int total_steps = sizeof(analysis_steps) / sizeof(analysis_steps[0]);
@@ -678,7 +736,7 @@ IAnalyzer::AddressCategory Analyzer::category(edb::address_t address) const {
 // Name: functions
 // Desc:
 //------------------------------------------------------------------------------
-IAnalyzer::FunctionMap Analyzer::functions(const IRegion::pointer &region) const {
+IAnalyzer::FunctionMap Analyzer::functions(const std::shared_ptr<IRegion> &region) const {
 	return analysis_info_[region->start()].functions;
 }
 
@@ -702,37 +760,70 @@ bool Analyzer::find_containing_function(edb::address_t address, Function *functi
 
 	Q_ASSERT(function);
 
-	if(IRegion::pointer region = edb::v1::memory_regions().find_region(address)) {
+	if(std::shared_ptr<IRegion> region = edb::v1::memory_regions().find_region(address)) {
 		const FunctionMap &funcs = functions(region);
-		for(const Function &f: funcs) {
-			if(address >= f.entry_address() && address <= f.end_address()) {
-				*function = f;
-				return true;
-			}
+
+		// upperBound returns the first item that is >= address here, or end().
+		// Note the ">", it does not do the opposite of lowerBound!
+		auto iter = funcs.upperBound(address);
+		if (iter == funcs.end()) {
+			return false;
+		}
+
+		// handle address == entrypoint case
+		if ((*iter).entry_address() == address) {
+			*function = *iter;
+			return true;
+		}
+
+		// go to previous function and test it
+		if (iter == funcs.begin()) {
+			return false;
+		}
+
+		const Function& func = *(--iter);
+		if (address >= func.entry_address() && address <= func.end_address()) {
+			*function = func;
+			return true;
 		}
 	}
 	return false;
 }
 
 //------------------------------------------------------------------------------
-// Name: md5_region
-// Desc: returns a byte array representing the MD5 of a region
+// Name: for_funcs_in_range
+// Desc: Calls functor once for every function that exists between the start and
+// end addresses. This includes functions whose bodies include the start address.
+// start and end must reside in the same region. If the functor returns false,
+// iteration is halted. Return value is true if all functions were iterated,
+// false if the iteration was halted early.
 //------------------------------------------------------------------------------
-QByteArray Analyzer::md5_region(const IRegion::pointer &region) const{
+bool Analyzer::for_funcs_in_range(const edb::address_t start, const edb::address_t end, std::function<bool(const Function*)> functor) const {
+	if (std::shared_ptr<IRegion> region = edb::v1::memory_regions().find_region(start)) {
+		const FunctionMap &funcs = functions(region);
+		auto it = funcs.lowerBound(start - 4096);
 
-	const edb::address_t page_size = edb::v1::debugger_core->page_size();
-	const size_t page_count        = region->size() / page_size;
+		while (it != funcs.end()) {
+			auto f_start = it->entry_address();
+			auto f_end = it->end_address();
 
-	const QVector<quint8> pages = edb::v1::read_pages(region->start(), page_count);
-	if(!pages.isEmpty()) {
-		return edb::v1::get_md5(pages);
+			// Only works if FunctionMap is a QMap
+			if (f_start > end) {
+				return true;
+			}
+			// ranges overlap: http://stackoverflow.com/a/3269471
+			if (f_start <= end && start <= f_end) {
+				if (!functor(&(*it))) {
+					return false;
+				}
+			}
+
+			++it;
+		}
 	}
-
-
-	return QByteArray();
+	return true;
 }
 
-
 //------------------------------------------------------------------------------
 // Name: bonus_entry_point
 // Desc:
@@ -761,9 +852,9 @@ void Analyzer::bonus_entry_point(RegionData *data) const {
 // Name: invalidate_analysis
 // Desc:
 //------------------------------------------------------------------------------
-void Analyzer::invalidate_analysis(const IRegion::pointer &region) {
+void Analyzer::invalidate_analysis(const std::shared_ptr<IRegion> &region) {
 	invalidate_dynamic_analysis(region);
-	for(const edb::address_t addr: specified_functions_) {
+	Q_FOREACH(const edb::address_t addr, specified_functions_) {
 		if(addr >= region->start() && addr < region->end()) {
 			specified_functions_.remove(addr);
 		}
@@ -774,7 +865,7 @@ void Analyzer::invalidate_analysis(const IRegion::pointer &region) {
 // Name: invalidate_dynamic_analysis
 // Desc:
 //------------------------------------------------------------------------------
-void Analyzer::invalidate_dynamic_analysis(const IRegion::pointer &region) {
+void Analyzer::invalidate_dynamic_analysis(const std::shared_ptr<IRegion> &region) {
 
 	RegionData info;
 	info.region = region;
@@ -812,21 +903,15 @@ Result<edb::address_t> Analyzer::find_containing_function(edb::address_t address
 //------------------------------------------------------------------------------
 bool Analyzer::will_return(edb::address_t address) const {
 
-	const QList<Symbol::pointer> symbols = edb::v1::symbol_manager().symbols();
-	for(const Symbol::pointer &symbol: symbols) {
-		if(symbol->address == address) {
-			const QString symname = symbol->name_no_prefix;
-			const QString func_name = symname.mid(0, symname.indexOf("@"));
-
-			if(func_name == "__assert_fail" || func_name == "abort" || func_name == "_exit" || func_name == "_Exit" || func_name == "_dl_signal_error" || func_name == "_dl_signal_cerror") {
+	const std::shared_ptr<Symbol> symbol = edb::v1::symbol_manager().find(address);
+	if(symbol) {
+		const QString symname = symbol->name_no_prefix;
+		const QString func_name = symname.mid(0, symname.indexOf("@"));
+		
+		if(const edb::Prototype *const info = edb::v1::get_function_info(func_name)) {
+			if(info->noreturn) {
 				return false;
 			}
-
-#ifdef Q_OS_LINUX
-			if(func_name == "_Unwind_Resume") {
-				return false;
-			}
-#endif
 		}
 	}
 
@@ -834,6 +919,64 @@ bool Analyzer::will_return(edb::address_t address) const {
 	return true;
 }
 
+//------------------------------------------------------------------------------
+// Name: get_analysis_path
+// Desc:
+//------------------------------------------------------------------------------
+
+QString Analyzer::get_analysis_path(const std::shared_ptr<IRegion> &region) const {
+	if (region->name().isEmpty()) {
+		return QString();
+	}
+
+	QString session_path = edb::v1::config().session_path;
+	if(session_path.isEmpty()) {
+		return QString();
+	}
+
+	// We need the base address of this region. However, this region might not
+	// be the same region that has the header. For instance, on Windows the
+	// PE header is not in the same region as the ELF header. The binary might
+	// also have multiple code sections.
+	edb::address_t base_address = 0;
+	edb::address_t loaded_address = 0;
+	{
+		QList<std::shared_ptr<IRegion>> regions = edb::v1::memory_regions().regions();
+		bool base_addr_found = false;
+		for(const std::shared_ptr<IRegion> &iregion: regions) {
+			if (iregion->name() == region->name()) {
+				if(auto binary_info = edb::v1::get_binary_info(iregion)) {
+					base_address = binary_info->base_address();
+					loaded_address = iregion->start();
+					base_addr_found = true;
+					break;
+				}
+			}
+		}
+
+		if (!base_addr_found) {
+			return QString();
+		}
+	}
+
+	QFileInfo info(region->name());
+
+	if(info.isRelative()) {
+		info.makeAbsolute();
+	}
+
+	auto path          = tr("%1/%2").arg(session_path, info.absolutePath());
+	const QString name = info.fileName();
+
+	// ensure that the sub-directory exists
+	QDir().mkpath(path);
+
+	return tr("%1/%2.Analysis.%3").arg(
+		path,
+		name,
+		QString::number(region->start() - loaded_address + base_address, 16));
+}
+
 #if QT_VERSION < 0x050000
 Q_EXPORT_PLUGIN2(Analyzer, Analyzer)
 #endif
