@@ -17,22 +17,25 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <QWidget>
+#include <QPixmap>
 
-namespace Analyzer {
+namespace AnalyzerPlugin {
 
 class AnalyzerWidget : public QWidget {
 	Q_OBJECT
 public:
 	AnalyzerWidget(QWidget *parent = 0, Qt::WindowFlags f = 0);
-	
+
 protected:
 	virtual void paintEvent(QPaintEvent *event);
 	virtual void mousePressEvent(QMouseEvent *event);
 	virtual void mouseReleaseEvent(QMouseEvent *event);
 	virtual void mouseMoveEvent(QMouseEvent *event);
-	
+
 private:
 	bool mouse_pressed_;
+	QPixmap* cache_;
+	int cache_num_funcs_;
 };
 
 }
