@@ -23,7 +23,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "Types.h"
 #include "IRegion.h"
 
-namespace Assembler {
+namespace AssemblerPlugin {
 
 namespace Ui { class DialogAssembler; }
 
@@ -33,10 +33,10 @@ class DialogAssembler : public QDialog {
 public:
 	DialogAssembler(QWidget *parent = 0);
 	virtual ~DialogAssembler();
-	
+
 public Q_SLOTS:
 	void on_buttonBox_accepted();
-	
+
 public:
 	void set_address(edb::address_t address);
 
