@@ -21,7 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QWidget>
 
-namespace Assembler {
+namespace AssemblerPlugin {
 
 namespace Ui { class OptionsPage; }
 
@@ -37,7 +37,7 @@ public:
 
 public Q_SLOTS:
 	virtual void on_assemblerName_currentIndexChanged(const QString &text);
-	
+
 private:
 	Ui::OptionsPage *const ui;
 };
