@@ -24,7 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 class QMenu;
 class QDialog;
 
-namespace Backtrace {
+namespace BacktracePlugin {
 
 class Backtrace : public QObject, public IPlugin
 {
@@ -47,8 +47,8 @@ public Q_SLOTS:
 	void show_menu();
 
 private:
-	QMenu	*menu_;
-	QDialog	*dialog_;
+	QMenu	         *menu_;
+	QPointer<QDialog> dialog_;
 };
 
 }
