@@ -24,7 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDialog>
 #include <QTableWidget>
 
-namespace Backtrace {
+namespace BacktracePlugin {
 
 namespace Ui {
 class DialogBacktrace;
