@@ -18,8 +18,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "DialogHeader.h"
 #include "edb.h"
-#include "ELF32.h"
-#include "ELF64.h"
+#include "ELFXX.h"
 #include "MemoryRegions.h"
 #include "PE32.h"
 #include <QMessageBox>
@@ -27,7 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "ui_DialogHeader.h"
 
-namespace BinaryInfo {
+namespace BinaryInfoPlugin {
 namespace {
 
 template <class T>
@@ -533,7 +532,7 @@ void DialogHeader::on_btnExplore_clicked() {
 		for(const QModelIndex &selected_item: sel) {
 
 			const QModelIndex index = filter_model_->mapToSource(selected_item);
-			if(auto region = *reinterpret_cast<const IRegion::pointer *>(index.internalPointer())) {
+			if(auto region = *reinterpret_cast<const std::shared_ptr<IRegion> *>(index.internalPointer())) {
 
 				if(auto binary_info = edb::v1::get_binary_info(region)) {
 
