@@ -22,24 +22,43 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "IBinary.h"
 #include "pe_binary.h"
 
-namespace BinaryInfo {
+namespace BinaryInfoPlugin {
+
+class PEBinaryException : public std::exception {
+	public:
+	enum reasonEnum {
+		INVALID_ARGUMENTS = 1,
+		READ_FAILURE = 2,
+		INVALID_PE = 3,
+		INVALID_ARCHITECTURE = 4
+	};
+		PEBinaryException(reasonEnum reason);
+
+		virtual const char * what() const noexcept override;
+
+	private:
+		reasonEnum reason_;
+};
 
 class PE32 : public IBinary {
 public:
-	PE32(const IRegion::pointer &region);
+	PE32(const std::shared_ptr<IRegion> &region);
 	virtual ~PE32();
 
 public:
 	virtual bool native() const;
-	virtual bool validate_header();
 	virtual edb::address_t calculate_main();
 	virtual edb::address_t debug_pointer();
 	virtual edb::address_t entry_point();
 	virtual size_t header_size() const;
 	virtual const void *header() const;
-	
+	virtual QVector<Header> headers() const;
+	virtual edb::address_t base_address() const;
+
 private:
-	IRegion::pointer region_;
+	std::shared_ptr<IRegion> region_;
+	IMAGE_DOS_HEADER dos_;
+	IMAGE_NT_HEADERS32 pe_;
 };
 
 }
