@@ -11,6 +11,8 @@
 #define DEMANGLING_SUPPORTED true
 
 inline QString demangle(const QString& mangled) {
+	if(!mangled.startsWith("_Z"))
+		return mangled; // otherwise we'll try to demangle C functions coinciding with types like "f" as "float", which is bad
 	int failed=0;
 	QStringList split=mangled.split("@"); // for cases like funcName@plt
 	std::unique_ptr<char,decltype(std::free)*> demangled(abi::__cxa_demangle(split.front().toStdString().c_str(),0,0,&failed), std::free);
