@@ -39,7 +39,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "elf/elf_shdr.h"
 #include "elf/elf_syminfo.h"
 
-namespace BinaryInfo {
+namespace BinaryInfoPlugin {
 namespace {
 
 struct elf32_model {
@@ -73,7 +73,7 @@ struct elf32_model {
 		}
 
 		QString to_string() const {
-			return QString("%1 %2 %3 %4").arg(edb::value32(address).toHexString()).arg(edb::value32(size).toHexString()).arg(type).arg(name);
+			return QString("%1 %2 %3 %4").arg(edb::value32(address).toHexString(), edb::value32(size).toHexString()).arg(type).arg(name);
 		}
 	};
 };
@@ -110,7 +110,7 @@ struct elf64_model {
 		}
 
 		QString to_string() const {
-			return QString("%1 %2 %3 %4").arg(edb::value64(address).toHexString()).arg(edb::value32(size).toHexString()).arg(type).arg(name);
+			return QString("%1 %2 %3 %4").arg(edb::value64(address).toHexString(), edb::value32(size).toHexString()).arg(type).arg(name);
 		}
 	};
 };
@@ -202,6 +202,9 @@ void collect_symbols(const void *p, size_t size, QList<typename M::symbol> &symb
 	const auto base = reinterpret_cast<uintptr_t>(p);
 
 	const auto header                              = static_cast<const elf_header_t*>(p);
+	if (header->e_shnum == 0 || header->e_shentsize == 0) {
+		return;
+	}
 	const auto sections_begin                      = reinterpret_cast<elf_section_header_t*>(base + header->e_shoff);
 	const elf_section_header_t *const sections_end = sections_begin + header->e_shnum;
 	auto section_strings                           = reinterpret_cast<const char*>(base + sections_begin[header->e_shstrndx].sh_offset);
@@ -241,6 +244,10 @@ void collect_symbols(const void *p, size_t size, QList<typename M::symbol> &symb
 				int n = 0;
 				auto relocation = reinterpret_cast<elf_relocation_a_t*>(base + section->sh_offset);
 
+                if(section->sh_link == 0) {
+                    break;
+                }
+
 				for(size_t i = 0; i < section->sh_size / section->sh_entsize; ++i) {
 
 					const int sym_index                = M::elf_r_sym(relocation[i].r_info);
@@ -274,6 +281,10 @@ void collect_symbols(const void *p, size_t size, QList<typename M::symbol> &symb
 				int n = 0;
 				auto relocation = reinterpret_cast<elf_relocation_t*>(base + section->sh_offset);
 
+                if(section->sh_link == 0) {
+                    break;
+                }
+
 				for(size_t i = 0; i < section->sh_size / section->sh_entsize; ++i) {
 
 					const int sym_index                = M::elf_r_sym(relocation[i].r_info);
@@ -396,7 +407,7 @@ void collect_symbols(const void *p, size_t size, QList<typename M::symbol> &symb
 
 //--------------------------------------------------------------------------
 // Name: output_symbols
-// Desc: outputs the symbols to OS ensuring uniqueness and adding any 
+// Desc: outputs the symbols to OS ensuring uniqueness and adding any
 //       needed demangling
 //--------------------------------------------------------------------------
 template <class Symbol>
@@ -420,10 +431,10 @@ void output_symbols(QList<Symbol> &symbols, std::ostream &os) {
 bool generate_symbols_internal(QFile &file, std::shared_ptr<QFile> &debugFile, std::ostream &os) {
 	if(auto file_ptr = reinterpret_cast<void *>(file.map(0, file.size(), QFile::NoOptions))) {
 		if(is_elf64(file_ptr)) {
-		
+
 			typedef typename elf64_model::symbol symbol;
 			QList<symbol> symbols;
-			
+
 			collect_symbols<elf64_model>(file_ptr, file.size(), symbols);
 
 			// if there was a debug file
@@ -440,16 +451,16 @@ bool generate_symbols_internal(QFile &file, std::shared_ptr<QFile> &debugFile, s
 						}
 					}
 				}
-			}			
+			}
 
 			output_symbols(symbols, os);
 			return true;
 		} else if(is_elf32(file_ptr)) {
-		
-			typedef typename elf32_model::symbol symbol;
-			QList<symbol> symbols;				
 
-			collect_symbols<elf32_model>(file_ptr, file.size(), symbols);			
+			typedef typename elf32_model::symbol symbol;
+			QList<symbol> symbols;
+
+			collect_symbols<elf32_model>(file_ptr, file.size(), symbols);
 
 			// if there was a debug file
 			if(debugFile) {
@@ -465,15 +476,15 @@ bool generate_symbols_internal(QFile &file, std::shared_ptr<QFile> &debugFile, s
 						}
 					}
 				}
-			}			
-			
+			}
+
 			output_symbols(symbols, os);
 			return true;
 		} else {
 			qDebug() << "unknown file type";
 		}
 	}
-	
+
 	return false;
 }
 
@@ -497,17 +508,17 @@ bool generate_symbols(const QString &filename, std::ostream &os) {
 		os << md5.toHex().data() << ' ' << qPrintable(QFileInfo(filename).absoluteFilePath()) << '\n';
 
 		const QString debugInfoPath = QSettings().value("BinaryInfo/debug_info_path", "/usr/lib/debug").toString();
-	
+
 		std::shared_ptr<QFile> debugFile;
 		if(!debugInfoPath.isEmpty()) {
 			debugFile = std::make_shared<QFile>(QString("%1/%2.debug").arg(debugInfoPath, filename));
 			if(!debugFile->exists()) // systems such as Ubuntu don't have .debug suffix, try without it
 				debugFile = std::make_shared<QFile>(QString("%1/%2").arg(debugInfoPath, filename));
 		}
-		
-		return generate_symbols_internal(file, debugFile, os); 
+
+		return generate_symbols_internal(file, debugFile, os);
 	}
-	
+
 	return false;
 }
 }
