@@ -24,7 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 class QMenu;
 class QDialog;
 
-namespace BinarySearcher {
+namespace BinarySearcherPlugin {
 
 class BinarySearcher : public QObject, public IPlugin {
 	Q_OBJECT
