@@ -23,7 +23,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 class QListWidgetItem;
 
-namespace BinarySearcher {
+namespace BinarySearcherPlugin {
 
 namespace Ui { class DialogASCIIString; }
 
