@@ -20,19 +20,23 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // TODO(eteran): research usage of process_vm_readv, process_vm_writev
 
 #include "DebuggerCore.h"
-#include "edb.h"
 #include "Configuration.h"
+#include "DialogMemoryAccess.h"
+#include "edb.h"
+#include "FeatureDetect.h"
 #include "MemoryRegions.h"
+#include "PlatformCommon.h"
 #include "PlatformEvent.h"
+#include "PlatformProcess.h"
 #include "PlatformRegion.h"
 #include "PlatformState.h"
-#include "PlatformProcess.h"
-#include "PlatformCommon.h"
+#include "PlatformThread.h"
 #include "State.h"
 #include "string_hash.h"
 
 #include <QDebug>
 #include <QDir>
+#include <QSettings>
 
 #include <cerrno>
 #include <cstring>
@@ -41,10 +45,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define _GNU_SOURCE        /* or _BSD_SOURCE or _SVID_SOURCE */
 #endif
 
+#if defined(EDB_X86) || defined(EDB_X86_64)
 #include <cpuid.h>
+#endif
+
 #include <sys/ptrace.h>
 #include <sys/mman.h>
 #include <sys/personality.h>
+#include <sys/wait.h>
 
 // doesn't always seem to be defined in the headers
 
@@ -64,7 +72,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define PTRACE_O_EXITKILL	(1 << 20)
 #endif
 
-namespace DebuggerCore {
+#ifndef PTRACE_O_TRACEEXIT
+#define PTRACE_O_TRACEEXIT	(1 << PTRACE_EVENT_EXIT)
+#endif
+
+namespace DebuggerCorePlugin {
 
 namespace {
 
@@ -89,13 +101,18 @@ bool is_numeric(const QString &s) {
 // Desc:
 //------------------------------------------------------------------------------
 bool is_clone_event(int status) {
-	if(WIFSTOPPED(status) && WSTOPSIG(status) == SIGTRAP) {
-		return (((status >> 16) & 0xffff) == PTRACE_EVENT_CLONE);
-	}
-
-	return false;
+    return (status >> 8 == (SIGTRAP | (PTRACE_EVENT_CLONE << 8)));
 }
 
+//------------------------------------------------------------------------------
+// Name: is_exit_trace_event
+// Desc:
+//------------------------------------------------------------------------------
+bool is_exit_trace_event(int status) {
+    return (status >> 8 == (SIGTRAP | (PTRACE_EVENT_EXIT << 8)));
+}
+
+#if defined(EDB_X86) || defined(EDB_X86_64)
 bool in64BitSegment() {
 	bool edbIsIn64BitSegment;
 	// Check that we're running in 64 bit segment: this can be in cases
@@ -110,9 +127,10 @@ bool in64BitSegment() {
 
 bool os64Bit(bool edbIsIn64BitSegment) {
 	bool osIs64Bit;
-	if(edbIsIn64BitSegment)
-		osIs64Bit=true;
-	else {
+
+	if(edbIsIn64BitSegment) {
+		osIs64Bit = true;
+	} else {
 		// We want to be really sure the OS is 32 bit, so we can't rely on such easy
 		// to (even unintentionally) fake mechanisms as uname(2) (e.g. see setarch(8))
 		asm(R"(.intel_syntax noprefix
@@ -124,9 +142,10 @@ bool os64Bit(bool edbIsIn64BitSegment) {
 			   .att_syntax # restore default syntax
 			   )":"=a"(osIs64Bit));
 	}
+
 	return osIs64Bit;
 }
-
+#endif
 
 }
 
@@ -134,18 +153,49 @@ bool os64Bit(bool edbIsIn64BitSegment) {
 // Name: DebuggerCore
 // Desc: constructor
 //------------------------------------------------------------------------------
-DebuggerCore::DebuggerCore() : 
-	binary_info_(nullptr),
-	process_(0),
+DebuggerCore::DebuggerCore() :
+	process_(nullptr),
 	pointer_size_(sizeof(void*)),
+#if defined(EDB_X86) || defined(EDB_X86_64)
 	edbIsIn64BitSegment(in64BitSegment()),
 	osIs64Bit(os64Bit(edbIsIn64BitSegment)),
-	USER_CS_32(osIs64Bit?0x23:0x73),
-	USER_CS_64(osIs64Bit?0x33:0xfff8), // RPL 0 can't appear in user segment registers, so 0xfff8 is safe
-	USER_SS(osIs64Bit?0x2b:0x7b)
-{
-	qDebug() << "EDB is in" << (edbIsIn64BitSegment?"64":"32") << "bit segment";
-	qDebug() << "OS is" << (osIs64Bit?"64":"32") << "bit";
+	USER_CS_32(osIs64Bit ? 0x23 : 0x73),
+	USER_CS_64(osIs64Bit ? 0x33 : 0xfff8), // RPL 0 can't appear in user segment registers, so 0xfff8 is safe
+	USER_SS(osIs64Bit    ? 0x2b : 0x7b),
+#endif
+	lastMeansOfCapture(MeansOfCapture::NeverCaptured)
+	
+	 {
+
+#if 0
+#if defined(EDB_X86) || defined(EDB_X86_64)
+	qDebug() << "EDB is in" << (edbIsIn64BitSegment ? "64" : "32") << "bit segment";
+	qDebug() << "OS is" << (osIs64Bit ? "64" : "32") << "bit";
+#endif
+#endif
+
+	proc_mem_write_broken_ = true;
+	proc_mem_read_broken_  = true;
+
+	feature::detect_proc_access(&proc_mem_read_broken_, &proc_mem_write_broken_);
+
+	if(proc_mem_read_broken_ || proc_mem_write_broken_) {
+
+		qDebug() << "Detect that read /proc/<pid>/mem works  = " << !proc_mem_read_broken_;
+		qDebug() << "Detect that write /proc/<pid>/mem works = " << !proc_mem_write_broken_;
+
+		QSettings settings;
+		const bool warn = settings.value("DebuggerCore/warn_on_broken_proc_mem.enabled", true).toBool();
+		if(warn) {
+			auto dialog = new DialogMemoryAccess(0);
+			dialog->exec();
+
+			settings.setValue("DebuggerCore/warn_on_broken_proc_mem.enabled", dialog->warnNextTime());
+
+			delete dialog;
+		}
+	}
+
 }
 
 //------------------------------------------------------------------------------
@@ -153,12 +203,17 @@ DebuggerCore::DebuggerCore() :
 // Desc:
 //------------------------------------------------------------------------------
 bool DebuggerCore::has_extension(quint64 ext) const {
+	
+	Q_UNUSED(ext);
+	
+#if defined(EDB_X86) || defined(EDB_X86_64)
+	static constexpr auto mmxHash = edb::string_hash("MMX");
+	static constexpr auto xmmHash = edb::string_hash("XMM");
+	static constexpr auto ymmHash = edb::string_hash("YMM");
 
-	const auto mmxHash=edb::string_hash("MMX");
-	const auto xmmHash=edb::string_hash("XMM");
-	const auto ymmHash=edb::string_hash("YMM");
-	if(EDB_IS_64_BIT && (ext==xmmHash || ext==mmxHash))
+	if(EDB_IS_64_BIT && (ext == xmmHash || ext == mmxHash)) {
 		return true;
+	}
 
 	quint32 eax;
 	quint32 ebx;
@@ -174,19 +229,23 @@ bool DebuggerCore::has_extension(quint64 ext) const {
 	case ymmHash:
 	{
 		// Check OSXSAVE and AVX feature flags
-		if((ecx&0x18000000)!=0x18000000)
+		if((ecx & 0x18000000) != 0x18000000) {
 			return false;
+		}
+
 		// Get XCR0, must be exactly after OSXSAVE feature check, otherwise #UD
-		asm volatile("xgetbv" : "=a"(eax),"=d"(edx) : "c"(0));
+		__asm__ __volatile__("xgetbv" : "=a"(eax),"=d"(edx) : "c"(0));
+
 		// Check that the OS has enabled XMM and YMM state support
-		if((eax&0x6)!=0x6)
+		if((eax & 0x6) != 0x6) {
 			return false;
+		}
 		return true;
 	}
 	default:
 		return false;
 	}
-
+#endif
 	return false;
 }
 
@@ -214,9 +273,14 @@ DebuggerCore::~DebuggerCore() {
 // Name: ptrace_getsiginfo
 // Desc:
 //------------------------------------------------------------------------------
-long DebuggerCore::ptrace_getsiginfo(edb::tid_t tid, siginfo_t *siginfo) {
+Status DebuggerCore::ptrace_getsiginfo(edb::tid_t tid, siginfo_t *siginfo) {
 	Q_ASSERT(siginfo != 0);
-	return ptrace(PTRACE_GETSIGINFO, tid, 0, siginfo);
+	if(ptrace(PTRACE_GETSIGINFO, tid, 0, siginfo)==-1) {
+		const char*const strError=strerror(errno);
+		qWarning() << "Unable to get signal info for thread" << tid << ": PTRACE_GETSIGINFO failed:" << strError;
+		return Status(strError);
+	}
+	return Status::Ok;
 }
 
 //------------------------------------------------------------------------------
@@ -231,71 +295,131 @@ long DebuggerCore::ptrace_traceme() {
 // Name: ptrace_continue
 // Desc:
 //------------------------------------------------------------------------------
-long DebuggerCore::ptrace_continue(edb::tid_t tid, long status) {
+Status DebuggerCore::ptrace_continue(edb::tid_t tid, long status) {
 	// TODO(eteran): perhaps address this at a higher layer?
-	//               I would like to not have these events show up 
+	//               I would like to not have these events show up
 	//               in the first place if we aren't stopped on this TID :-(
 	if(waited_threads_.contains(tid)) {
-		Q_ASSERT(waited_threads_.contains(tid));
 		Q_ASSERT(tid != 0);
+		if(ptrace(PTRACE_CONT, tid, 0, status)==-1) {
+			const char*const strError=strerror(errno);
+			qWarning() << "Unable to continue thread" << tid << ": PTRACE_CONT failed:" << strError;
+			return Status(strError);
+		}
 		waited_threads_.remove(tid);
-		return ptrace(PTRACE_CONT, tid, 0, status);
+		return Status::Ok;
 	}
-	return -1;
+	return Status(QObject::tr("ptrace_continue(): waited_threads_ doesn't contain tid %1").arg(tid));
 }
 
 //------------------------------------------------------------------------------
 // Name: ptrace_step
 // Desc:
 //------------------------------------------------------------------------------
-long DebuggerCore::ptrace_step(edb::tid_t tid, long status) {
+Status DebuggerCore::ptrace_step(edb::tid_t tid, long status) {
 	// TODO(eteran): perhaps address this at a higher layer?
-	//               I would like to not have these events show up 
+	//               I would like to not have these events show up
 	//               in the first place if we aren't stopped on this TID :-(
 	if(waited_threads_.contains(tid)) {
-		Q_ASSERT(waited_threads_.contains(tid));
 		Q_ASSERT(tid != 0);
+		if(ptrace(PTRACE_SINGLESTEP, tid, 0, status)==-1) {
+			const char*const strError=strerror(errno);
+			qWarning() << "Unable to step thread" << tid << ": PTRACE_SINGLESTEP failed:" << strError;
+			return Status(strError);
+		}
 		waited_threads_.remove(tid);
-		return ptrace(PTRACE_SINGLESTEP, tid, 0, status);
+		return Status::Ok;
 	}
-	return -1;
+	return Status(QObject::tr("ptrace_step(): waited_threads_ doesn't contain tid %1").arg(tid));
 }
 
 //------------------------------------------------------------------------------
 // Name: ptrace_set_options
 // Desc:
 //------------------------------------------------------------------------------
-long DebuggerCore::ptrace_set_options(edb::tid_t tid, long options) {
+Status DebuggerCore::ptrace_set_options(edb::tid_t tid, long options) {
 	Q_ASSERT(waited_threads_.contains(tid));
 	Q_ASSERT(tid != 0);
-	return ptrace(PTRACE_SETOPTIONS, tid, 0, options);
+	if(ptrace(PTRACE_SETOPTIONS, tid, 0, options)==-1) {
+		const char*const strError=strerror(errno);
+		qWarning() << "Unable to set ptrace options for thread" << tid << ": PTRACE_SETOPTIONS failed:" << strError;
+		return Status(strError);
+	}
+	return Status::Ok;
 }
 
 //------------------------------------------------------------------------------
 // Name: ptrace_get_event_message
 // Desc:
 //------------------------------------------------------------------------------
-long DebuggerCore::ptrace_get_event_message(edb::tid_t tid, unsigned long *message) {
+Status DebuggerCore::ptrace_get_event_message(edb::tid_t tid, unsigned long *message) {
 	Q_ASSERT(waited_threads_.contains(tid));
 	Q_ASSERT(tid != 0);
 	Q_ASSERT(message != 0);
-	return ptrace(PTRACE_GETEVENTMSG, tid, 0, message);
+	if(ptrace(PTRACE_GETEVENTMSG, tid, 0, message)==-1) {
+		const char*const strError=strerror(errno);
+		qWarning() << "Unable to get event message for thread" << tid << ": PTRACE_GETEVENTMSG failed:" << strError;
+		return Status(strError);
+	}
+	return Status::Ok;
+}
+
+//------------------------------------------------------------------------------
+// Name: desired_ptrace_options
+// Desc:
+//------------------------------------------------------------------------------
+long DebuggerCore::ptraceOptions() const {
+
+    // we want to trace clone (thread) creation events
+    long options = PTRACE_O_TRACECLONE;
+
+    // if applicable, we want an auto SIGKILL sent to the child
+    // process and its threads
+    switch(edb::v1::config().close_behavior) {
+    case Configuration::Kill:
+        options |= PTRACE_O_EXITKILL;
+        break;
+    case Configuration::KillIfLaunchedDetachIfAttached:
+        if(last_means_of_capture() == MeansOfCapture::Launch) {
+            options |= PTRACE_O_EXITKILL;
+        }
+        break;
+    default:
+        break;
+    }
+
+#if 0
+    // TODO(eteran): research this option for issue #46
+    options |= PTRACE_O_TRACEEXIT;
+#endif
+
+    return options;
+}
+
+//------------------------------------------------------------------------------
+// Name: handle_thread_exit
+// Desc:
+//------------------------------------------------------------------------------
+void DebuggerCore::handle_thread_exit(edb::tid_t tid, int status) {
+	Q_UNUSED(status);
+
+	threads_.remove(tid);
+	waited_threads_.remove(tid);
 }
 
 //------------------------------------------------------------------------------
 // Name: handle_event
 // Desc:
 //------------------------------------------------------------------------------
-IDebugEvent::const_pointer DebuggerCore::handle_event(edb::tid_t tid, int status) {
+std::shared_ptr<IDebugEvent> DebuggerCore::handle_event(edb::tid_t tid, int status) {
 
 	// note that we have waited on this thread
 	waited_threads_.insert(tid);
 
 	// was it a thread exit event?
 	if(WIFEXITED(status)) {
-		threads_.remove(tid);
-		waited_threads_.remove(tid);
 
+		handle_thread_exit(tid,status);
 		// if this was the last thread, return true
 		// so we report it to the user.
 		// if this wasn't, then we should silently
@@ -305,11 +429,15 @@ IDebugEvent::const_pointer DebuggerCore::handle_event(edb::tid_t tid, int status
 		}
 	}
 
+    if(is_exit_trace_event(status)) {
+
+    }
+
 	// was it a thread create event?
 	if(is_clone_event(status)) {
 
 		unsigned long new_tid;
-		if(ptrace_get_event_message(tid, &new_tid) != -1) {
+		if(ptrace_get_event_message(tid, &new_tid)) {
 
 			auto newThread            = std::make_shared<PlatformThread>(this, process_, new_tid);
 			newThread->status_        = 0;
@@ -324,10 +452,16 @@ IDebugEvent::const_pointer DebuggerCore::handle_event(edb::tid_t tid, int status
 				}
 			}
 
-			if(!WIFSTOPPED(thread_status) || WSTOPSIG(thread_status) != SIGSTOP) {
-				qDebug("handle_event(): [warning] new thread [%d] received an event besides SIGSTOP: status=0x%x", static_cast<int>(new_tid),thread_status);
+			// A new thread could exit before we have fully created it, no event then since it can't be the last thread
+			if(WIFEXITED(thread_status)) {
+				handle_thread_exit(tid,thread_status);
+				return nullptr;
 			}
-			
+
+			if(!WIFSTOPPED(thread_status) || WSTOPSIG(thread_status) != SIGSTOP) {
+				qWarning("handle_event(): new thread [%d] received an event besides SIGSTOP: status=0x%x", static_cast<int>(new_tid),thread_status);
+			}
+
 			newThread->status_ = thread_status;
 
 			// copy the hardware debug registers from the current thread to the new thread
@@ -350,25 +484,54 @@ IDebugEvent::const_pointer DebuggerCore::handle_event(edb::tid_t tid, int status
 	}
 
 	// normal event
-
-
 	auto e = std::make_shared<PlatformEvent>();
 
 	e->pid_    = pid();
 	e->tid_    = tid;
 	e->status_ = status;
-	if(ptrace_getsiginfo(tid, &e->siginfo_) == -1) {
+	if(!ptrace_getsiginfo(tid, &e->siginfo_)) {
 		// TODO: handle no info?
 	}
 
 	active_thread_ = tid;
-	
+
 	auto it = threads_.find(tid);
 	if(it != threads_.end()) {
 		it.value()->status_ = status;
 	}
 
 	stop_threads();
+
+	// Some breakpoint types result in SIGILL or SIGSEGV. We'll transform the
+	// event into breakpoint event if such a breakpoint has triggered.
+	if(it != threads_.end() && WIFSTOPPED(status)) {
+		const auto signo=WSTOPSIG(status);
+		if(signo==SIGILL || signo==SIGSEGV) {
+			// no need to peekuser for SIGILL, but have to for SIGSEGV
+			const auto address = signo==SIGILL ? edb::address_t::fromZeroExtended(e->siginfo_.si_addr)
+											   : (*it)->instruction_pointer();
+			if(edb::v1::find_triggered_breakpoint(address))
+			{
+				e->status_=SIGTRAP<<8|0x7f;
+				e->siginfo_.si_signo=SIGTRAP;
+				e->siginfo_.si_code=TRAP_BRKPT;
+			}
+		}
+	}
+
+#if defined EDB_ARM32
+	if(it != threads_.end()) {
+		const auto& thread=*it;
+		if(thread->singleStepBreakpoint) {
+
+			remove_breakpoint(thread->singleStepBreakpoint->address());
+			thread->singleStepBreakpoint=nullptr;
+
+			assert(e->siginfo_.si_signo==SIGTRAP); // signo must have already be converted to SIGTRAP if needed
+			e->siginfo_.si_code=TRAP_TRACE;
+		}
+	}
+#endif
 	return e;
 }
 
@@ -376,30 +539,44 @@ IDebugEvent::const_pointer DebuggerCore::handle_event(edb::tid_t tid, int status
 // Name: stop_threads
 // Desc:
 //------------------------------------------------------------------------------
-void DebuggerCore::stop_threads() {
+Status DebuggerCore::stop_threads() {
+
+	QString errorMessage;
+
 	if(process_) {
 		for(auto &thread: process_->threads()) {
 			const edb::tid_t tid = thread->tid();
 
 			if(!waited_threads_.contains(tid)) {
-			
+
 				if(auto thread_ptr = std::static_pointer_cast<PlatformThread>(thread)) {
-			
-					thread->stop();
+
+					const auto stopStatus=thread->stop();
+					if(!stopStatus)
+						errorMessage+=QObject::tr("Failed to stop thread %1: %2\n").arg(tid).arg(stopStatus.toString());
 
 					int thread_status;
 					if(native::waitpid(tid, &thread_status, __WALL) > 0) {
 						waited_threads_.insert(tid);
 						thread_ptr->status_ = thread_status;
 
-						if(!WIFSTOPPED(thread_status) || WSTOPSIG(thread_status) != SIGSTOP) {
-							qDebug("stop_threads(): [warning] paused thread [%d] received an event besides SIGSTOP: status=0x%x", tid,thread_status);
+						// A thread could have exited between previous waitpid and the latest one...
+						if(WIFEXITED(thread_status)) {
+							handle_thread_exit(tid, thread_status);
+						}
+						// ..., otherwise it must have stopped.
+						else if(!WIFSTOPPED(thread_status) || WSTOPSIG(thread_status) != SIGSTOP) {
+							qWarning("stop_threads(): paused thread [%d] received an event besides SIGSTOP: status=0x%x", tid,thread_status);
 						}
 					}
 				}
 			}
 		}
 	}
+	if(errorMessage.isEmpty())
+		return Status::Ok;
+	qWarning() << errorMessage.toStdString().c_str();
+	return Status("\n"+errorMessage);
 }
 
 //------------------------------------------------------------------------------
@@ -407,11 +584,11 @@ void DebuggerCore::stop_threads() {
 // Desc: waits for a debug event, msecs is a timeout
 //      it will return false if an error or timeout occurs
 //------------------------------------------------------------------------------
-IDebugEvent::const_pointer DebuggerCore::wait_debug_event(int msecs) {
+std::shared_ptr<IDebugEvent> DebuggerCore::wait_debug_event(int msecs) {
 
 	if(process_) {
 		if(!native::wait_for_sigchld(msecs)) {
-			for(auto thread : process_->threads()) {
+			for(auto &thread : process_->threads()) {
 				int status;
 				const edb::tid_t tid = native::waitpid(thread->tid(), &status, __WALL | WNOHANG);
 				if(tid > 0) {
@@ -432,7 +609,7 @@ int DebuggerCore::attach_thread(edb::tid_t tid) {
 		// I *think* that the PTRACE_O_TRACECLONE is only valid on
 		// stopped threads
 		int status;
-		const auto ret=native::waitpid(tid, &status, __WALL);
+        const auto ret = native::waitpid(tid, &status, __WALL);
 		if(ret > 0) {
 
 			auto newThread            = std::make_shared<PlatformThread>(this, process_, tid);
@@ -442,45 +619,46 @@ int DebuggerCore::attach_thread(edb::tid_t tid) {
 			threads_[tid] = newThread;
 
 			waited_threads_.insert(tid);
-			if(ptrace_set_options(tid, PTRACE_O_TRACECLONE) == -1) {
-				qDebug("[DebuggerCore] failed to set PTRACE_O_TRACECLONE: [%d] %s", tid, strerror(errno));
-			}
-			
-			if(edb::v1::config().close_behavior==Configuration::Kill ||
-			   (edb::v1::config().close_behavior==Configuration::KillIfLaunchedDetachIfAttached &&
-				  last_means_of_capture()==MeansOfCapture::Launch)) {
-				if(ptrace_set_options(tid, PTRACE_O_EXITKILL) == -1) {
-					qDebug("[DebuggerCore] failed to set PTRACE_O_EXITKILL: [%d] %s", tid, strerror(errno));
-				}
-			}
+
+            const long options = ptraceOptions();
+
+            const auto setoptStatus=ptrace_set_options(tid, options);
+			if(!setoptStatus)	{
+                qDebug() << "[DebuggerCore] failed to set ptrace options: ["<< tid <<"]" << setoptStatus.toString();
+            }
+
 			return 0;
-		}
-		else if(ret==-1) {
+        } else if(ret==-1) {
 			return errno;
-		}
-		else return -1; // unknown error
-	}
-	else return errno;
+        } else {
+            return -1; // unknown error
+        }
+    } else {
+        return errno;
+    }
 }
 
 //------------------------------------------------------------------------------
 // Name: attach
 // Desc:
 //------------------------------------------------------------------------------
-QString DebuggerCore::attach(edb::pid_t pid) {
-
-	static const QString statusOK{};
+Status DebuggerCore::attach(edb::pid_t pid) {
 
 	end_debug_session();
 
-	lastMeansOfCapture=MeansOfCapture::Attach;
-	
-	// create this, so the threads created can refer to it
-	process_ = new PlatformProcess(this, pid);
+	lastMeansOfCapture = MeansOfCapture::Attach;
 
-	int lastErr=attach_thread(pid); // Fail early if we are going to
-	if(lastErr) return std::strerror(lastErr);
-	lastErr=-2;
+	// create this, so the threads created can refer to it
+    process_ = new PlatformProcess(this, pid);
+
+	int lastErr = attach_thread(pid); // Fail early if we are going to
+	if(lastErr) {
+        delete process_;
+        process_ = nullptr;
+		return Status(std::strerror(lastErr));
+	}
+
+	lastErr = -2;
 	bool attached;
 	do {
 		attached = false;
@@ -491,11 +669,11 @@ QString DebuggerCore::attach(edb::pid_t pid) {
 			// all in one shot
 			const edb::tid_t tid = s.toUInt();
 			if(!threads_.contains(tid)) {
-				const auto errnum=attach_thread(tid);
-				if(errnum==0)
+                const auto errnum = attach_thread(tid);
+				if(errnum == 0)
 					attached = true;
 				else
-					lastErr=errnum;
+					lastErr = errnum;
 			}
 		}
 	} while(attached);
@@ -504,37 +682,44 @@ QString DebuggerCore::attach(edb::pid_t pid) {
 	if(!threads_.empty()) {
 		pid_            = pid;
 		active_thread_  = pid;
-		binary_info_    = edb::v1::get_binary_info(edb::v1::primary_code_region());		
-		detectDebuggeeBitness();
-		return statusOK;
-	} else {
-		delete process_;
-		process_ = nullptr;
+		binary_info_    = edb::v1::get_binary_info(edb::v1::primary_code_region());
+		detectCPUMode();
+		return Status::Ok;
 	}
 
-	return std::strerror(lastErr);
+    delete process_;
+    process_ = nullptr;
+	return Status(std::strerror(lastErr));
 }
 
 //------------------------------------------------------------------------------
 // Name: detach
 // Desc:
 //------------------------------------------------------------------------------
-void DebuggerCore::detach() {
+Status DebuggerCore::detach() {
+	QString errorMessage;
 	if(process_) {
 
 		stop_threads();
 
 		clear_breakpoints();
 
-		for(auto thread: process_->threads()) {
-			ptrace(PTRACE_DETACH, thread->tid(), 0, 0);
+		for(auto &thread: process_->threads()) {
+			if(ptrace(PTRACE_DETACH, thread->tid(), 0, 0)==-1) {
+				const char*const strError=strerror(errno);
+				errorMessage+=QObject::tr("Unable to detach from thread %1: PTRACE_DETACH failed: %2\n").arg(thread->tid()).arg(strError);
+			}
 		}
-		
+
 		delete process_;
 		process_ = nullptr;
 
 		reset();
 	}
+	if(errorMessage.isEmpty())
+		return Status::Ok;
+	qWarning() << errorMessage.toStdString().c_str();
+	return Status(errorMessage);
 }
 
 //------------------------------------------------------------------------------
@@ -547,18 +732,19 @@ void DebuggerCore::kill() {
 
 		::kill(pid(), SIGKILL);
 
-		// TODO: do i need to actually do this wait?
-		native::waitpid(pid(), 0, __WALL);
+		pid_t ret;
+		while((ret=native::waitpid(-1, 0, __WALL)) != pid() && ret!=-1);
 
 		delete process_;
-		process_ = 0;
+		process_ = nullptr;
 
 		reset();
 	}
 }
 
-void DebuggerCore::detectDebuggeeBitness() {
+void DebuggerCore::detectCPUMode() {
 
+#if defined(EDB_X86) || defined(EDB_X86_64)
 	const size_t offset=EDB_IS_64_BIT ?
 						offsetof(UserRegsStructX86_64, cs) :
 						offsetof(UserRegsStructX86,   xcs);
@@ -568,19 +754,45 @@ void DebuggerCore::detectDebuggeeBitness() {
 		if(cs==USER_CS_32) {
 			if(pointer_size_==sizeof(quint64)) {
 				qDebug() << "Debuggee is now 32 bit";
-				CapstoneEDB::init(false);
+				cpu_mode_=CPUMode::x86_32;
+				CapstoneEDB::init(CapstoneEDB::Architecture::ARCH_X86);
 			}
 			pointer_size_=sizeof(quint32);
 			return;
 		} else if(cs==USER_CS_64) {
 			if(pointer_size_==sizeof(quint32)) {
 				qDebug() << "Debuggee is now 64 bit";
-				CapstoneEDB::init(true);
+				cpu_mode_=CPUMode::x86_64;
+				CapstoneEDB::init(CapstoneEDB::Architecture::ARCH_AMD64);
 			}
 			pointer_size_=sizeof(quint64);
 			return;
 		}
 	}
+#elif defined(EDB_ARM32)
+	errno=0;
+	const auto cpsr=ptrace(PTRACE_PEEKUSER, active_thread_, sizeof(long)*16, 0L);
+	if(!errno) {
+		const bool thumb=cpsr&0x20;
+		if(thumb)
+		{
+			cpu_mode_=CPUMode::Thumb;
+			CapstoneEDB::init(CapstoneEDB::Architecture::ARCH_ARM32_THUMB);
+		}
+		else
+		{
+			cpu_mode_=CPUMode::ARM32;
+			CapstoneEDB::init(CapstoneEDB::Architecture::ARCH_ARM32_ARM);
+		}
+	}
+	pointer_size_ = sizeof(quint32);
+#elif defined(EDB_ARM64)
+	cpu_mode_=CPUMode::ARM64;
+	CapstoneEDB::init(CapstoneEDB::Architecture::ARCH_ARM64);
+	pointer_size_ = sizeof(quint64);
+#else
+	#error "Unsupported Architecture"
+#endif
 }
 
 //------------------------------------------------------------------------------
@@ -588,9 +800,9 @@ void DebuggerCore::detectDebuggeeBitness() {
 // Desc:
 //------------------------------------------------------------------------------
 void DebuggerCore::get_state(State *state) {
-	// TODO: assert that we are paused	
+	// TODO: assert that we are paused
 	if(process_) {
-		if(IThread::pointer thread = process_->current_thread()) {
+		if(std::shared_ptr<IThread> thread = process_->current_thread()) {
 			thread->get_state(state);
 		}
 	}
@@ -604,7 +816,7 @@ void DebuggerCore::set_state(const State &state) {
 
 	// TODO: assert that we are paused
 	if(process_) {
-		if(IThread::pointer thread = process_->current_thread()) {
+		if(std::shared_ptr<IThread> thread = process_->current_thread()) {
 			thread->set_state(state);
 		}
 	}
@@ -614,13 +826,11 @@ void DebuggerCore::set_state(const State &state) {
 // Name: open
 // Desc:
 //------------------------------------------------------------------------------
-QString DebuggerCore::open(const QString &path, const QString &cwd, const QList<QByteArray> &args, const QString &tty) {
-
-	static const QString statusOK{};
+Status DebuggerCore::open(const QString &path, const QString &cwd, const QList<QByteArray> &args, const QString &tty) {
 
 	end_debug_session();
 
-	lastMeansOfCapture=MeansOfCapture::Launch;
+    lastMeansOfCapture = MeansOfCapture::Launch;
 
 	static constexpr std::size_t sharedMemSize=4096;
 	const auto sharedMem=static_cast<QChar*>(::mmap(nullptr,sharedMemSize,PROT_READ|PROT_WRITE,MAP_SHARED|MAP_ANONYMOUS,-1,0));
@@ -646,24 +856,27 @@ QString DebuggerCore::open(const QString &path, const QString &cwd, const QList<
 		}
 
 		if(edb::v1::config().disableASLR) {
-			const auto curPers=::personality(UINT32_MAX);
+            const auto curPers = ::personality(UINT32_MAX);
 			// This shouldn't fail, but let's at least perror if it does anyway
-			if(curPers==-1)
+            if(curPers == -1) {
 				perror("Failed to get current personality");
-			else if(::personality(curPers|ADDR_NO_RANDOMIZE)==-1)
+            } else if(::personality(curPers | ADDR_NO_RANDOMIZE) == -1) {
 				perror("Failed to disable ASLR");
+            }
 		}
 
-		if(edb::v1::config().disableLazyBinding && setenv("LD_BIND_NOW","1",true)==-1)
+        if(edb::v1::config().disableLazyBinding && setenv("LD_BIND_NOW", "1",true) == -1) {
 			perror("Failed to disable lazy binding");
+        }
 
 		// do the actual exec
-		const QString error=execute_process(path, cwd, args);
-#if defined __GNUG__ && __GNUC__ >= 5 || !defined __GNUG__ || \
-		defined __clang__ && __clang_major__*100+__clang_minor__>=306
+		const Status status = execute_process(path, cwd, args);
+
+#if defined __GNUG__ && __GNUC__ >= 5 || !defined __GNUG__ || defined __clang__ && __clang_major__*100+__clang_minor__>=306
 		static_assert(std::is_trivially_copyable<QChar>::value,"Can't copy string of QChar to shared memory");
 #endif
-		std::memcpy(sharedMem,error.constData(),std::min(sizeof(QChar)*error.size(),sharedMemSize-sizeof(QChar)/*prevent overwriting of last null*/));
+		QString error = status.toString();
+		std::memcpy(sharedMem, error.constData(), std::min(sizeof(QChar) * error.size(), sharedMemSize-sizeof(QChar)/*prevent overwriting of last null*/));
 
 		// we should never get here!
 		abort();
@@ -672,74 +885,69 @@ QString DebuggerCore::open(const QString &path, const QString &cwd, const QList<
 	case -1:
 		// error! for some reason we couldn't fork
 		reset();
-		return QObject::tr("Failed to fork");
+		return Status(tr("Failed to fork"));
 	default:
 		// parent
 		do {
 			reset();
 
 			int status;
-			const auto wpidRet=native::waitpid(pid, &status, __WALL);
+            const auto wpidRet = native::waitpid(pid, &status, __WALL);
 			const QString childError(sharedMem);
-			::munmap(sharedMem,sharedMemSize);
+            ::munmap(sharedMem, sharedMemSize);
 
-			if(wpidRet == -1)
-				return QObject::tr("waitpid() failed: %1").arg(std::strerror(errno))+(childError.isEmpty()?"":QObject::tr(".\nError returned by child:\n%1.").arg(childError));
+            if(wpidRet == -1) {
+				return Status(tr("waitpid() failed: %1").arg(std::strerror(errno))+(childError.isEmpty()?"" : tr(".\nError returned by child:\n%1.").arg(childError)));
+            }
 
-			if(WIFEXITED(status))
-				return QObject::tr("The child unexpectedly exited with code %1. Error returned by child:\n%2").arg(WEXITSTATUS(status)).arg(childError);
-			if(WIFSIGNALED(status))
-				return QObject::tr("The child was unexpectedly killed by signal %1. Error returned by child:\n%2").arg(WTERMSIG(status)).arg(childError);
+            if(WIFEXITED(status)) {
+				return Status(tr("The child unexpectedly exited with code %1. Error returned by child:\n%2").arg(WEXITSTATUS(status)).arg(childError));
+            }
+
+            if(WIFSIGNALED(status)) {
+				return Status(tr("The child was unexpectedly killed by signal %1. Error returned by child:\n%2").arg(WTERMSIG(status)).arg(childError));
+            }
 
 			// This happens when exec failed, but just in case it's something another return some description.
-			if(WIFSTOPPED(status) && WSTOPSIG(status) == SIGABRT)
-				return childError.isEmpty() ? QObject::tr("The child unexpectedly aborted") : childError;
+            if(WIFSTOPPED(status) && WSTOPSIG(status) == SIGABRT) {
+				return Status(childError.isEmpty() ? tr("The child unexpectedly aborted") : childError);
+            }
 
 			// the very first event should be a STOP of type SIGTRAP
 			if(!WIFSTOPPED(status) || WSTOPSIG(status) != SIGTRAP) {
 				end_debug_session();
-				return QObject::tr("First event after waitpid() should be a STOP of type SIGTRAP, but wasn't, instead status=0x%1")
-											.arg(status,0,16)+(childError.isEmpty()?"":QObject::tr(".\nError returned by child:\n%1.").arg(childError));
+				return Status(tr("First event after waitpid() should be a STOP of type SIGTRAP, but wasn't, instead status=0x%1")
+											.arg(status,0,16)+(childError.isEmpty() ? "" : tr(".\nError returned by child:\n%1.").arg(childError)));
 			}
 
-			waited_threads_.insert(pid);
+            waited_threads_.insert(pid);
 
-			// enable following clones (threads)
-			if(ptrace_set_options(pid, PTRACE_O_TRACECLONE) == -1) {
-				const auto strerr=strerror(errno); // NOTE: must be called before end_debug_session, otherwise errno can change
-				end_debug_session();
-				return QObject::tr("[DebuggerCore] failed to set PTRACE_O_TRACECLONE: %1").arg(strerr);
-			}
-			
-#ifdef PTRACE_O_EXITKILL
-			if(ptrace_set_options(pid, PTRACE_O_EXITKILL) == -1) {
-				const auto strerr=strerror(errno); // NOTE: must be called before any other syscall, otherwise errno can change
-				// Don't consider the error fatal: the option is only supported since Linux 3.8
-				qDebug() << "[DebuggerCore] failed to set PTRACE_O_EXITKILL:" << strerr;
-			}
-#endif
+            const long options = ptraceOptions();
 
-			// setup the first event data for the primary thread
-			waited_threads_.insert(pid);
-			
-			// create the process
+            // enable following clones (threads) and other options we are concerned with
+			const auto setoptStatus=ptrace_set_options(pid, options);
+            if(!setoptStatus) {
+                end_debug_session();
+                return Status(tr("[DebuggerCore] failed to set ptrace options: %1").arg(setoptStatus.toString()));
+            }
+
+            // create the process
 			process_ = new PlatformProcess(this, pid);
-			
 
 			// the PID == primary TID
 			auto newThread            = std::make_shared<PlatformThread>(this, process_, pid);
 			newThread->status_        = status;
 			newThread->signal_status_ = PlatformThread::Stopped;
-			
+
 			threads_[pid]   = newThread;
 
 			pid_            = pid;
-			active_thread_   = pid;
+            active_thread_  = pid;
 			binary_info_    = edb::v1::get_binary_info(edb::v1::primary_code_region());
 
-			detectDebuggeeBitness();
+			detectCPUMode();
 
-			return statusOK;
+			return Status::Ok;
 		} while(0);
 		break;
 	}
@@ -750,7 +958,7 @@ QString DebuggerCore::open(const QString &path, const QString &cwd, const QList<
 // Name: last_means_of_capture
 // Desc: Returns how the last process was captured to debug
 //------------------------------------------------------------------------------
-DebuggerCore::MeansOfCapture DebuggerCore::last_means_of_capture() {
+DebuggerCore::MeansOfCapture DebuggerCore::last_means_of_capture() const {
 	return lastMeansOfCapture;
 }
 
@@ -778,8 +986,8 @@ IState *DebuggerCore::create_state() const {
 // Name: enumerate_processes
 // Desc:
 //------------------------------------------------------------------------------
-QMap<edb::pid_t, IProcess::pointer> DebuggerCore::enumerate_processes() const {
-	QMap<edb::pid_t, IProcess::pointer> ret;
+QMap<edb::pid_t, std::shared_ptr<IProcess>> DebuggerCore::enumerate_processes() const {
+	QMap<edb::pid_t, std::shared_ptr<IProcess>> ret;
 
 	QDir proc_directory("/proc/");
 	QFileInfoList entries = proc_directory.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
@@ -788,7 +996,7 @@ QMap<edb::pid_t, IProcess::pointer> DebuggerCore::enumerate_processes() const {
 		const QString filename = info.fileName();
 		if(is_numeric(filename)) {
 			const edb::pid_t pid = filename.toULong();
-			
+
 			// NOTE(eteran): the const_cast is reasonable here.
 			// While we don't want THIS function to mutate the DebuggerCore object
 			// we do want the associated PlatformProcess to be able to trigger
@@ -821,10 +1029,18 @@ edb::pid_t DebuggerCore::parent_pid(edb::pid_t pid) const {
 // Desc: Returns EDB's native CPU type
 //------------------------------------------------------------------------------
 quint64 DebuggerCore::cpu_type() const {
+#if defined EDB_X86 || defined EDB_X86_64
 	if(EDB_IS_32_BIT)
 		return edb::string_hash("x86");
 	else
 		return edb::string_hash("x86-64");
+#elif defined EDB_ARM32
+	return edb::string_hash("arm");
+#elif defined EDB_ARM64
+	return edb::string_hash("AArch64");
+#else
+#	error "What to return?"
+#endif
 }
 
 
@@ -841,10 +1057,16 @@ QString DebuggerCore::format_pointer(edb::address_t address) const {
 // Desc:
 //------------------------------------------------------------------------------
 QString DebuggerCore::stack_pointer() const {
+#if defined EDB_X86 || defined EDB_X86_64
 	if(edb::v1::debuggeeIs32Bit())
 		return "esp";
 	else
 		return "rsp";
+#elif defined EDB_ARM32 || defined EDB_ARM64
+	return "sp";
+#else
+#	error "What to return?"
+#endif
 }
 
 //------------------------------------------------------------------------------
@@ -852,10 +1074,16 @@ QString DebuggerCore::stack_pointer() const {
 // Desc:
 //------------------------------------------------------------------------------
 QString DebuggerCore::frame_pointer() const {
+#if defined EDB_X86 || defined EDB_X86_64
 	if(edb::v1::debuggeeIs32Bit())
 		return "ebp";
 	else
 		return "rbp";
+#elif defined EDB_ARM32 || defined EDB_ARM64
+	return "fp";
+#else
+#	error "What to return?"
+#endif
 }
 
 //------------------------------------------------------------------------------
@@ -863,10 +1091,16 @@ QString DebuggerCore::frame_pointer() const {
 // Desc:
 //------------------------------------------------------------------------------
 QString DebuggerCore::instruction_pointer() const {
+#if defined EDB_X86 || defined EDB_X86_64
 	if(edb::v1::debuggeeIs32Bit())
 		return "eip";
 	else
 		return "rip";
+#elif defined EDB_ARM32 || defined EDB_ARM64
+	return "pc";
+#else
+#	error "What to return?"
+#endif
 }
 
 //------------------------------------------------------------------------------
@@ -874,15 +1108,21 @@ QString DebuggerCore::instruction_pointer() const {
 // Desc: Returns the name of the flag register as a QString.
 //------------------------------------------------------------------------------
 QString DebuggerCore::flag_register() const {
+#if defined EDB_X86 || defined EDB_X86_64
 	if(edb::v1::debuggeeIs32Bit())
 		return "eflags";
 	else
 		return "rflags";
+#elif defined EDB_ARM32 || defined EDB_ARM64
+	return "cpsr";
+#else
+#	error "What to return?"
+#endif
 }
 
 //------------------------------------------------------------------------------
 // Name: process
-// Desc: 
+// Desc:
 //------------------------------------------------------------------------------
 IProcess *DebuggerCore::process() const {
 	return process_;
