@@ -200,7 +200,7 @@ DebuggerCore::~DebuggerCore() {
 // Desc: waits for a debug event, msecs is a timeout
 //      it will return false if an error or timeout occurs
 //------------------------------------------------------------------------------
-IDebugEvent::const_pointer DebuggerCore::wait_debug_event(int msecs) {
+std::shared_ptr<const IDebugEvent> DebuggerCore::wait_debug_event(int msecs) {
 	if(attached()) {
 		int status;
 		bool timeout;
@@ -568,9 +568,9 @@ edb::pid_t DebuggerCore::parent_pid(edb::pid_t pid) const {
 // Name:
 // Desc:
 //------------------------------------------------------------------------------
-QList<IRegion::pointer> DebuggerCore::memory_regions() const {
+QList<std::shared_ptr<IRegion>> DebuggerCore::memory_regions() const {
 
-	QList<IRegion::pointer> regions;
+	QList<std::shared_ptr<IRegion>> regions;
 
 	if(pid_ != 0) {
 
@@ -630,7 +630,7 @@ do_unload:
 			kvm_close(kd);
 		} else {
 			fprintf(stderr, "sync: %s\n", err_buf);
-			return QList<IRegion::pointer>();
+			return QList<std::shared_ptr<IRegion>>();
 		}
 	}
 
