@@ -21,7 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <limits>
 #include <cmath>
 
-namespace DebuggerCore {
+namespace DebuggerCorePlugin {
 
 namespace {
 
@@ -130,119 +130,119 @@ Register PlatformState::value(const QString &reg) const {
 	const QString lreg = reg.toLower();
 
 #if defined(EDB_X86)
-	if(lreg == "eax")          return Register("eax", context_.Eax, Register::TYPE_GPR);
-	else if(lreg == "ebx")     return Register("ebx", context_.Ebx, Register::TYPE_GPR);
-	else if(lreg == "ecx")     return Register("ecx", context_.Ecx, Register::TYPE_GPR);
-	else if(lreg == "edx")     return Register("edx", context_.Edx, Register::TYPE_GPR);
-	else if(lreg == "ebp")     return Register("ebp", context_.Ebp, Register::TYPE_GPR);
-	else if(lreg == "esp")     return Register("esp", context_.Esp, Register::TYPE_GPR);
-	else if(lreg == "esi")     return Register("esi", context_.Esi, Register::TYPE_GPR);
-	else if(lreg == "edi")     return Register("edi", context_.Edi, Register::TYPE_GPR);
-	else if(lreg == "eip")     return Register("eip", context_.Eip, Register::TYPE_IP);
-	else if(lreg == "ax")      return Register("ax", context_.Eax & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "bx")      return Register("bx", context_.Ebx & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "cx")      return Register("cx", context_.Ecx & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "dx")      return Register("dx", context_.Edx & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "bp")      return Register("bp", context_.Ebp & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "sp")      return Register("sp", context_.Esp & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "si")      return Register("si", context_.Esi & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "di")      return Register("di", context_.Edi & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "al")      return Register("al", context_.Eax & 0xff, Register::TYPE_GPR);
-	else if(lreg == "bl")      return Register("bl", context_.Ebx & 0xff, Register::TYPE_GPR);
-	else if(lreg == "cl")      return Register("cl", context_.Ecx & 0xff, Register::TYPE_GPR);
-	else if(lreg == "dl")      return Register("dl", context_.Edx & 0xff, Register::TYPE_GPR);
-	else if(lreg == "ah")      return Register("ah", (context_.Eax >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "bh")      return Register("bh", (context_.Ebx >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "ch")      return Register("ch", (context_.Ecx >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "dh")      return Register("dh", (context_.Edx >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "cs")      return Register("cs", context_.SegCs, Register::TYPE_SEG);
-	else if(lreg == "ds")      return Register("ds", context_.SegDs, Register::TYPE_SEG);
-	else if(lreg == "es")      return Register("es", context_.SegEs, Register::TYPE_SEG);
-	else if(lreg == "fs")      return Register("fs", context_.SegFs, Register::TYPE_SEG);
-	else if(lreg == "gs")      return Register("gs", context_.SegGs, Register::TYPE_SEG);
-	else if(lreg == "ss")      return Register("ss", context_.SegSs, Register::TYPE_SEG);
-	else if(lreg == "fs_base") return Register("fs_base", fs_base_, Register::TYPE_SEG);
-	else if(lreg == "gs_base") return Register("gs_base", gs_base_, Register::TYPE_SEG);
-	else if(lreg == "eflags")  return Register("eflags", context_.EFlags, Register::TYPE_COND);
+	if(lreg == "eax")          return make_Register("eax", context_.Eax, Register::TYPE_GPR);
+	else if(lreg == "ebx")     return make_Register("ebx", context_.Ebx, Register::TYPE_GPR);
+	else if(lreg == "ecx")     return make_Register("ecx", context_.Ecx, Register::TYPE_GPR);
+	else if(lreg == "edx")     return make_Register("edx", context_.Edx, Register::TYPE_GPR);
+	else if(lreg == "ebp")     return make_Register("ebp", context_.Ebp, Register::TYPE_GPR);
+	else if(lreg == "esp")     return make_Register("esp", context_.Esp, Register::TYPE_GPR);
+	else if(lreg == "esi")     return make_Register("esi", context_.Esi, Register::TYPE_GPR);
+	else if(lreg == "edi")     return make_Register("edi", context_.Edi, Register::TYPE_GPR);
+	else if(lreg == "eip")     return make_Register("eip", context_.Eip, Register::TYPE_IP);
+	else if(lreg == "ax")      return make_Register("ax", context_.Eax & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "bx")      return make_Register("bx", context_.Ebx & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "cx")      return make_Register("cx", context_.Ecx & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "dx")      return make_Register("dx", context_.Edx & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "bp")      return make_Register("bp", context_.Ebp & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "sp")      return make_Register("sp", context_.Esp & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "si")      return make_Register("si", context_.Esi & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "di")      return make_Register("di", context_.Edi & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "al")      return make_Register("al", context_.Eax & 0xff, Register::TYPE_GPR);
+	else if(lreg == "bl")      return make_Register("bl", context_.Ebx & 0xff, Register::TYPE_GPR);
+	else if(lreg == "cl")      return make_Register("cl", context_.Ecx & 0xff, Register::TYPE_GPR);
+	else if(lreg == "dl")      return make_Register("dl", context_.Edx & 0xff, Register::TYPE_GPR);
+	else if(lreg == "ah")      return make_Register("ah", (context_.Eax >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "bh")      return make_Register("bh", (context_.Ebx >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "ch")      return make_Register("ch", (context_.Ecx >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "dh")      return make_Register("dh", (context_.Edx >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "cs")      return make_Register("cs", context_.SegCs, Register::TYPE_SEG);
+	else if(lreg == "ds")      return make_Register("ds", context_.SegDs, Register::TYPE_SEG);
+	else if(lreg == "es")      return make_Register("es", context_.SegEs, Register::TYPE_SEG);
+	else if(lreg == "fs")      return make_Register("fs", context_.SegFs, Register::TYPE_SEG);
+	else if(lreg == "gs")      return make_Register("gs", context_.SegGs, Register::TYPE_SEG);
+	else if(lreg == "ss")      return make_Register("ss", context_.SegSs, Register::TYPE_SEG);
+	else if(lreg == "fs_base") return make_Register("fs_base", fs_base_, Register::TYPE_SEG);
+	else if(lreg == "gs_base") return make_Register("gs_base", gs_base_, Register::TYPE_SEG);
+	else if(lreg == "eflags")  return make_Register("eflags", context_.EFlags, Register::TYPE_COND);
 #elif defined(EDB_X86_64)
-	if(lreg == "rax")          return Register("rax", context_.Rax, Register::TYPE_GPR);
-	else if(lreg == "rbx")     return Register("rbx", context_.Rbx, Register::TYPE_GPR);
-	else if(lreg == "rcx")     return Register("rcx", context_.Rcx, Register::TYPE_GPR);
-	else if(lreg == "rdx")     return Register("rdx", context_.Rdx, Register::TYPE_GPR);
-	else if(lreg == "rbp")     return Register("rbp", context_.Rbp, Register::TYPE_GPR);
-	else if(lreg == "rsp")     return Register("rsp", context_.Rsp, Register::TYPE_GPR);
-	else if(lreg == "rsi")     return Register("rsi", context_.Rsi, Register::TYPE_GPR);
-	else if(lreg == "rdi")     return Register("rdi", context_.Rdi, Register::TYPE_GPR);
-	else if(lreg == "rip")     return Register("rip", context_.Rip, Register::TYPE_IP);
-	else if(lreg == "r8")      return Register("r8", context_.R8, Register::TYPE_GPR);
-	else if(lreg == "r9")      return Register("r9", context_.R9, Register::TYPE_GPR);
-	else if(lreg == "r10")     return Register("r10", context_.R10, Register::TYPE_GPR);
-	else if(lreg == "r11")     return Register("r11", context_.R11, Register::TYPE_GPR);
-	else if(lreg == "r12")     return Register("r12", context_.R12, Register::TYPE_GPR);
-	else if(lreg == "r13")     return Register("r13", context_.R13, Register::TYPE_GPR);
-	else if(lreg == "r14")     return Register("r14", context_.R14, Register::TYPE_GPR);
-	else if(lreg == "r15")     return Register("r15", context_.R15, Register::TYPE_GPR);
-	else if(lreg == "eax")     return Register("eax", context_.Rax & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "ebx")     return Register("ebx", context_.Rbx & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "ecx")     return Register("ecx", context_.Rcx & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "edx")     return Register("edx", context_.Rdx & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "ebp")     return Register("ebp", context_.Rbp & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "esp")     return Register("esp", context_.Rsp & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "esi")     return Register("esi", context_.Rsi & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "edi")     return Register("edi", context_.Rdi & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "r8d")     return Register("r8d", context_.R8 & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "r9d")     return Register("r9d", context_.R9 & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "r10d")    return Register("r10d", context_.R10 & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "r11d")    return Register("r11d", context_.R11 & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "r12d")    return Register("r12d", context_.R12 & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "r13d")    return Register("r13d", context_.R13 & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "r14d")    return Register("r14d", context_.R14 & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "r15d")    return Register("r15d", context_.R15 & 0xffffffff, Register::TYPE_GPR);
-	else if(lreg == "ax")      return Register("ax", context_.Rax & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "bx")      return Register("bx", context_.Rbx & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "cx")      return Register("cx", context_.Rcx & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "dx")      return Register("dx", context_.Rdx & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "bp")      return Register("bp", context_.Rbp & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "sp")      return Register("sp", context_.Rsp & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "si")      return Register("si", context_.Rsi & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "di")      return Register("di", context_.Rdi & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "r8w")     return Register("r8w", context_.R8 & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "r9w")     return Register("r9w", context_.R9 & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "r10w")    return Register("r10w", context_.R10 & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "r11w")    return Register("r11w", context_.R11 & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "r12w")    return Register("r12w", context_.R12 & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "r13w")    return Register("r13w", context_.R13 & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "r14w")    return Register("r14w", context_.R14 & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "r15w")    return Register("r15w", context_.R15 & 0xffff, Register::TYPE_GPR);
-	else if(lreg == "al")      return Register("al", context_.Rax & 0xff, Register::TYPE_GPR);
-	else if(lreg == "bl")      return Register("bl", context_.Rbx & 0xff, Register::TYPE_GPR);
-	else if(lreg == "cl")      return Register("cl", context_.Rcx & 0xff, Register::TYPE_GPR);
-	else if(lreg == "dl")      return Register("dl", context_.Rdx & 0xff, Register::TYPE_GPR);
-	else if(lreg == "ah")      return Register("ah", (context_.Rax >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "bh")      return Register("bh", (context_.Rbx >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "ch")      return Register("ch", (context_.Rcx >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "dh")      return Register("dh", (context_.Rdx >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "spl")     return Register("spl", (context_.Rsp >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "bpl")     return Register("bpl", (context_.Rbp >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "sil")     return Register("sil", (context_.Rsi >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "dil")     return Register("dil", (context_.Rdi >> 8) & 0xff, Register::TYPE_GPR);
-	else if(lreg == "r8b")     return Register("r8b", context_.R8 & 0xff, Register::TYPE_GPR);
-	else if(lreg == "r9b")     return Register("r9b", context_.R9 & 0xff, Register::TYPE_GPR);
-	else if(lreg == "r10b")    return Register("r10b", context_.R10 & 0xff, Register::TYPE_GPR);
-	else if(lreg == "r11b")    return Register("r11b", context_.R11 & 0xff, Register::TYPE_GPR);
-	else if(lreg == "r12b")    return Register("r12b", context_.R12 & 0xff, Register::TYPE_GPR);
-	else if(lreg == "r13b")    return Register("r13b", context_.R13 & 0xff, Register::TYPE_GPR);
-	else if(lreg == "r14b")    return Register("r14b", context_.R14 & 0xff, Register::TYPE_GPR);
-	else if(lreg == "r15b")    return Register("r15b", context_.R15 & 0xff, Register::TYPE_GPR);
-	else if(lreg == "cs")      return Register("cs", context_.SegCs, Register::TYPE_SEG);
-	else if(lreg == "ds")      return Register("ds", context_.SegDs, Register::TYPE_SEG);
-	else if(lreg == "es")      return Register("es", context_.SegEs, Register::TYPE_SEG);
-	else if(lreg == "fs")      return Register("fs", context_.SegFs, Register::TYPE_SEG);
-	else if(lreg == "gs")      return Register("gs", context_.SegGs, Register::TYPE_SEG);
-	else if(lreg == "ss")      return Register("ss", context_.SegSs, Register::TYPE_SEG);
-	else if(lreg == "fs_base") return Register("fs_base", fs_base_, Register::TYPE_SEG);
-	else if(lreg == "gs_base") return Register("gs_base", gs_base_, Register::TYPE_SEG);
-	else if(lreg == "rflags")  return Register("rflags", context_.EFlags, Register::TYPE_COND);
+	if(lreg == "rax")          return make_Register("rax", context_.Rax, Register::TYPE_GPR);
+	else if(lreg == "rbx")     return make_Register("rbx", context_.Rbx, Register::TYPE_GPR);
+	else if(lreg == "rcx")     return make_Register("rcx", context_.Rcx, Register::TYPE_GPR);
+	else if(lreg == "rdx")     return make_Register("rdx", context_.Rdx, Register::TYPE_GPR);
+	else if(lreg == "rbp")     return make_Register("rbp", context_.Rbp, Register::TYPE_GPR);
+	else if(lreg == "rsp")     return make_Register("rsp", context_.Rsp, Register::TYPE_GPR);
+	else if(lreg == "rsi")     return make_Register("rsi", context_.Rsi, Register::TYPE_GPR);
+	else if(lreg == "rdi")     return make_Register("rdi", context_.Rdi, Register::TYPE_GPR);
+	else if(lreg == "rip")     return make_Register("rip", context_.Rip, Register::TYPE_IP);
+	else if(lreg == "r8")      return make_Register("r8", context_.R8, Register::TYPE_GPR);
+	else if(lreg == "r9")      return make_Register("r9", context_.R9, Register::TYPE_GPR);
+	else if(lreg == "r10")     return make_Register("r10", context_.R10, Register::TYPE_GPR);
+	else if(lreg == "r11")     return make_Register("r11", context_.R11, Register::TYPE_GPR);
+	else if(lreg == "r12")     return make_Register("r12", context_.R12, Register::TYPE_GPR);
+	else if(lreg == "r13")     return make_Register("r13", context_.R13, Register::TYPE_GPR);
+	else if(lreg == "r14")     return make_Register("r14", context_.R14, Register::TYPE_GPR);
+	else if(lreg == "r15")     return make_Register("r15", context_.R15, Register::TYPE_GPR);
+	else if(lreg == "eax")     return make_Register("eax", context_.Rax & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "ebx")     return make_Register("ebx", context_.Rbx & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "ecx")     return make_Register("ecx", context_.Rcx & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "edx")     return make_Register("edx", context_.Rdx & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "ebp")     return make_Register("ebp", context_.Rbp & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "esp")     return make_Register("esp", context_.Rsp & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "esi")     return make_Register("esi", context_.Rsi & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "edi")     return make_Register("edi", context_.Rdi & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "r8d")     return make_Register("r8d", context_.R8 & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "r9d")     return make_Register("r9d", context_.R9 & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "r10d")    return make_Register("r10d", context_.R10 & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "r11d")    return make_Register("r11d", context_.R11 & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "r12d")    return make_Register("r12d", context_.R12 & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "r13d")    return make_Register("r13d", context_.R13 & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "r14d")    return make_Register("r14d", context_.R14 & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "r15d")    return make_Register("r15d", context_.R15 & 0xffffffff, Register::TYPE_GPR);
+	else if(lreg == "ax")      return make_Register("ax", context_.Rax & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "bx")      return make_Register("bx", context_.Rbx & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "cx")      return make_Register("cx", context_.Rcx & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "dx")      return make_Register("dx", context_.Rdx & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "bp")      return make_Register("bp", context_.Rbp & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "sp")      return make_Register("sp", context_.Rsp & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "si")      return make_Register("si", context_.Rsi & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "di")      return make_Register("di", context_.Rdi & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "r8w")     return make_Register("r8w", context_.R8 & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "r9w")     return make_Register("r9w", context_.R9 & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "r10w")    return make_Register("r10w", context_.R10 & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "r11w")    return make_Register("r11w", context_.R11 & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "r12w")    return make_Register("r12w", context_.R12 & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "r13w")    return make_Register("r13w", context_.R13 & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "r14w")    return make_Register("r14w", context_.R14 & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "r15w")    return make_Register("r15w", context_.R15 & 0xffff, Register::TYPE_GPR);
+	else if(lreg == "al")      return make_Register("al", context_.Rax & 0xff, Register::TYPE_GPR);
+	else if(lreg == "bl")      return make_Register("bl", context_.Rbx & 0xff, Register::TYPE_GPR);
+	else if(lreg == "cl")      return make_Register("cl", context_.Rcx & 0xff, Register::TYPE_GPR);
+	else if(lreg == "dl")      return make_Register("dl", context_.Rdx & 0xff, Register::TYPE_GPR);
+	else if(lreg == "ah")      return make_Register("ah", (context_.Rax >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "bh")      return make_Register("bh", (context_.Rbx >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "ch")      return make_Register("ch", (context_.Rcx >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "dh")      return make_Register("dh", (context_.Rdx >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "spl")     return make_Register("spl", (context_.Rsp >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "bpl")     return make_Register("bpl", (context_.Rbp >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "sil")     return make_Register("sil", (context_.Rsi >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "dil")     return make_Register("dil", (context_.Rdi >> 8) & 0xff, Register::TYPE_GPR);
+	else if(lreg == "r8b")     return make_Register("r8b", context_.R8 & 0xff, Register::TYPE_GPR);
+	else if(lreg == "r9b")     return make_Register("r9b", context_.R9 & 0xff, Register::TYPE_GPR);
+	else if(lreg == "r10b")    return make_Register("r10b", context_.R10 & 0xff, Register::TYPE_GPR);
+	else if(lreg == "r11b")    return make_Register("r11b", context_.R11 & 0xff, Register::TYPE_GPR);
+	else if(lreg == "r12b")    return make_Register("r12b", context_.R12 & 0xff, Register::TYPE_GPR);
+	else if(lreg == "r13b")    return make_Register("r13b", context_.R13 & 0xff, Register::TYPE_GPR);
+	else if(lreg == "r14b")    return make_Register("r14b", context_.R14 & 0xff, Register::TYPE_GPR);
+	else if(lreg == "r15b")    return make_Register("r15b", context_.R15 & 0xff, Register::TYPE_GPR);
+	else if(lreg == "cs")      return make_Register("cs", context_.SegCs, Register::TYPE_SEG);
+	else if(lreg == "ds")      return make_Register("ds", context_.SegDs, Register::TYPE_SEG);
+	else if(lreg == "es")      return make_Register("es", context_.SegEs, Register::TYPE_SEG);
+	else if(lreg == "fs")      return make_Register("fs", context_.SegFs, Register::TYPE_SEG);
+	else if(lreg == "gs")      return make_Register("gs", context_.SegGs, Register::TYPE_SEG);
+	else if(lreg == "ss")      return make_Register("ss", context_.SegSs, Register::TYPE_SEG);
+	else if(lreg == "fs_base") return make_Register("fs_base", fs_base_, Register::TYPE_SEG);
+	else if(lreg == "gs_base") return make_Register("gs_base", gs_base_, Register::TYPE_SEG);
+	else if(lreg == "rflags")  return make_Register("rflags", context_.EFlags, Register::TYPE_COND);
 #endif
 
 	return Register();
@@ -288,7 +288,7 @@ edb::address_t PlatformState::stack_pointer() const {
 // Name: debug_register
 // Desc:
 //------------------------------------------------------------------------------
-edb::reg_t PlatformState::debug_register(int n) const {
+edb::reg_t PlatformState::debug_register(size_t n) const {
 
 	switch(n) {
 	case 0: return context_.Dr0;
@@ -410,7 +410,7 @@ void PlatformState::clear() {
 // Name: set_debug_register
 // Desc:
 //------------------------------------------------------------------------------
-void PlatformState::set_debug_register(int n, edb::reg_t value) {
+void PlatformState::set_debug_register(size_t n, edb::reg_t value) {
 	switch(n) {
 	case 0: context_.Dr0 = value; break;
 	case 1: context_.Dr1 = value; break;
