@@ -24,7 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 class QMenu;
 class QDialog;
 
-namespace FunctionFinder {
+namespace FunctionFinderPlugin {
 
 class FunctionFinder : public QObject, public IPlugin {
 	Q_OBJECT
@@ -47,7 +47,7 @@ public Q_SLOTS:
 
 private:
 	QMenu *   menu_;
-	QDialog * dialog_;
+	QPointer<QDialog> dialog_;
 };
 
 }
