@@ -19,8 +19,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "HardwareBreakpoints.h"
 #include "edb.h"
 #include "IDebugger.h"
+#include "IProcess.h"
+#include "IDebugEvent.h"
 #include "DialogHWBreakpoints.h"
 #include "State.h"
+#include "IThread.h"
 
 #include <QMenu>
 #include <QDialog>
@@ -37,13 +40,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #error "Unsupported Platform"
 #endif
 
-namespace HardwareBreakpoints {
+namespace HardwareBreakpointsPlugin {
 
 //------------------------------------------------------------------------------
 // Name: HardwareBreakpoints
 // Desc:
 //------------------------------------------------------------------------------
-HardwareBreakpoints::HardwareBreakpoints() : menu_(0), dialog_(0), old_event_handler_(0) {
+HardwareBreakpoints::HardwareBreakpoints() : menu_(0), dialog_(0) {
 	auto dialog = new DialogHWBreakpoints(edb::v1::debugger_ui);
 	dialog_ = dialog;
 
@@ -95,6 +98,14 @@ void HardwareBreakpoints::setupBreakpoints() {
 
 	if(IProcess *process = edb::v1::debugger_core->process()) {
 
+		if(!process->isPaused()) {
+			QMessageBox::warning(
+				nullptr,
+				tr("Process Not Paused"),
+				tr("Unable to update hardware breakpoints because the process does not appear to be currently paused. Please suspend the process."));
+			return;
+		}
+
 		const bool enabled =
 			enabled_[Register1]->isChecked() ||
 			enabled_[Register2]->isChecked() ||
@@ -114,7 +125,7 @@ void HardwareBreakpoints::setupBreakpoints() {
 
 			if(!ok[Register1] && !ok[Register2] && !ok[Register3] && !ok[Register4]) {
 				QMessageBox::critical(
-					0,
+					nullptr,
 					tr("Address Error"),
 					tr("An address expression provided does not appear to be valid"));
 				return;
@@ -122,56 +133,52 @@ void HardwareBreakpoints::setupBreakpoints() {
 
 			for(int i = 0; i < RegisterCount; ++i) {
 				if(ok[i]) {
-				
+
 					const BreakpointStatus status = validateBreakpoint({
-						enabled_[i]->isChecked(), 
-						addr[i], 
-						types_[i]->currentIndex(), 
+						enabled_[i]->isChecked(),
+						addr[i],
+						types_[i]->currentIndex(),
 						sizes_[i]->currentIndex()
 					});
-					
+
 					switch(status) {
 					case AlignmentError:
 						QMessageBox::critical(
-							0,
+							nullptr,
 							tr("Address Alignment Error"),
-							tr("Hardware read/write breakpoint address must be aligned to breakpoint size."));					
+							tr("Hardware read/write breakpoint address must be aligned to breakpoint size."));
 						return;
 					case SizeError:
 					QMessageBox::critical(
-						0,
+						nullptr,
 						tr("BP Size Error"),
-						tr("Hardware read/write breakpoints cannot be 8-bytes in a 32-bit debuggee."));					
+						tr("Hardware read/write breakpoints cannot be 8-bytes in a 32-bit debuggee."));
 						return;
 					case Valid:
 						break;
 					}
 				}
 			}
-			
+
 			// we want to be enabled, if we aren't already hooked,
 			// hook it
-			if(!old_event_handler_) {
-				old_event_handler_ = edb::v1::set_debug_event_handler(this);
-			}
+			edb::v1::add_debug_event_handler(this);
 
-			for(IThread::pointer thread : process->threads()) {
+			for(std::shared_ptr<IThread> &thread : process->threads()) {
 				State state;
 				thread->get_state(&state);
 
-
 				for(int i = 0; i < RegisterCount; ++i) {
 					if(ok[i]) {
 						setBreakpointState(
-							&state, 
-							i, 
+							&state,
+							i,
 							{
-								enabled_[i]->isChecked(), 
-								addr[i], 
-								types_[i]->currentIndex(), 
+								enabled_[i]->isChecked(),
+								addr[i],
+								types_[i]->currentIndex(),
 								sizes_[i]->currentIndex()
-							}
-							);
+							});
 					}
 				}
 
@@ -180,7 +187,7 @@ void HardwareBreakpoints::setupBreakpoints() {
 
 		} else {
 
-			for(IThread::pointer thread : process->threads()) {
+			for(std::shared_ptr<IThread> &thread : process->threads()) {
 				State state;
 				thread->get_state(&state);
 				state.set_debug_register(7, 0);
@@ -188,10 +195,7 @@ void HardwareBreakpoints::setupBreakpoints() {
 			}
 
 			// we want to be disabled and we have hooked, so unhook
-			if(old_event_handler_) {
-				edb::v1::set_debug_event_handler(old_event_handler_);
-				old_event_handler_ = 0;
-			}
+			edb::v1::remove_debug_event_handler(this);
 		}
 	}
 
@@ -215,12 +219,12 @@ void HardwareBreakpoints::show_menu() {
 // Desc: this hooks the debug event handler so we can make the breakpoints
 //       able to be resumed
 //------------------------------------------------------------------------------
-edb::EVENT_STATUS HardwareBreakpoints::handle_event(const IDebugEvent::const_pointer &event) {
+edb::EVENT_STATUS HardwareBreakpoints::handle_event(const std::shared_ptr<IDebugEvent> &event) {
 
 	if(event->stopped() && event->is_trap()) {
 
 		if(IProcess *process = edb::v1::debugger_core->process()) {
-			if(IThread::pointer thread = process->current_thread()) {
+			if(std::shared_ptr<IThread> thread = process->current_thread()) {
 				// check DR6 to see if it was a HW BP event
 				// if so, set the resume flag
 				State state;
@@ -234,7 +238,7 @@ edb::EVENT_STATUS HardwareBreakpoints::handle_event(const IDebugEvent::const_poi
 	}
 
 	// pass the event down the stack
-	return old_event_handler_->handle_event(event);
+	return edb::DEBUG_NEXT_HANDLER;
 }
 
 //------------------------------------------------------------------------------
@@ -356,17 +360,23 @@ QList<QAction *> HardwareBreakpoints::cpu_context_menu() {
 
 //------------------------------------------------------------------------------
 // Name: setExecuteBP
-// Desc: 
+// Desc:
 //------------------------------------------------------------------------------
 void HardwareBreakpoints::setExecuteBP(int index, bool inUse) {
 	// we want to be enabled, if we aren't already hooked,
 	// hook it
-	if(!old_event_handler_) {
-		old_event_handler_ = edb::v1::set_debug_event_handler(this);
-	}
+	edb::v1::add_debug_event_handler(this);
 
 	if(IProcess *process = edb::v1::debugger_core->process()) {
 
+		if(!process->isPaused()) {
+			QMessageBox::warning(
+				nullptr,
+				tr("Process Not Paused"),
+				tr("Unable to update hardware breakpoints because the process does not appear to be currently paused. Please suspend the process."));
+			return;
+		}
+
 		if(inUse) {
 			QMessageBox::StandardButton button = QMessageBox::question(nullptr, tr("Breakpoint Already In Use"), tr("This breakpoint is already being used. Do you want to replace it?"), QMessageBox::Yes | QMessageBox::Cancel);
 			if(button != QMessageBox::Yes) {
@@ -376,7 +386,7 @@ void HardwareBreakpoints::setExecuteBP(int index, bool inUse) {
 
 		edb::address_t address = edb::v1::cpu_selected_address();
 
-		for(IThread::pointer thread : process->threads()) {
+		for(std::shared_ptr<IThread> &thread : process->threads()) {
 			State state;
 			thread->get_state(&state);
 			setBreakpointState(&state, index, { true, address, 0, 0 });
@@ -389,17 +399,23 @@ void HardwareBreakpoints::setExecuteBP(int index, bool inUse) {
 
 //------------------------------------------------------------------------------
 // Name: setWriteBP
-// Desc: 
+// Desc:
 //------------------------------------------------------------------------------
 void HardwareBreakpoints::setWriteBP(int index, bool inUse, edb::address_t address, int size) {
 	// we want to be enabled, if we aren't already hooked,
 	// hook it
-	if(!old_event_handler_) {
-		old_event_handler_ = edb::v1::set_debug_event_handler(this);
-	}
-	
+	edb::v1::add_debug_event_handler(this);
+
 	if(IProcess *process = edb::v1::debugger_core->process()) {
 
+		if(!process->isPaused()) {
+			QMessageBox::warning(
+				nullptr,
+				tr("Process Not Paused"),
+				tr("Unable to update hardware breakpoints because the process does not appear to be currently paused. Please suspend the process."));
+			return;
+		}
+
 		if(inUse) {
 			QMessageBox::StandardButton button = QMessageBox::question(nullptr, tr("Breakpoint Already In Use"), tr("This breakpoint is already being used. Do you want to replace it?"), QMessageBox::Yes | QMessageBox::Cancel);
 			if(button != QMessageBox::Yes) {
@@ -407,7 +423,7 @@ void HardwareBreakpoints::setWriteBP(int index, bool inUse, edb::address_t addre
 			}
 		}
 
-		for(IThread::pointer thread : process->threads()) {
+		for(std::shared_ptr<IThread> &thread : process->threads()) {
 			State state;
 			thread->get_state(&state);
 
@@ -438,17 +454,23 @@ void HardwareBreakpoints::setWriteBP(int index, bool inUse, edb::address_t addre
 
 //------------------------------------------------------------------------------
 // Name: setReadWriteBP
-// Desc: 
+// Desc:
 //------------------------------------------------------------------------------
 void HardwareBreakpoints::setReadWriteBP(int index, bool inUse, edb::address_t address, int size) {
 	// we want to be enabled, if we aren't already hooked,
 	// hook it
-	if(!old_event_handler_) {
-		old_event_handler_ = edb::v1::set_debug_event_handler(this);
-	}
-	
+	edb::v1::add_debug_event_handler(this);
+
 	if(IProcess *process = edb::v1::debugger_core->process()) {
 
+		if(!process->isPaused()) {
+			QMessageBox::warning(
+				nullptr,
+				tr("Process Not Paused"),
+				tr("Unable to update hardware breakpoints because the process does not appear to be currently paused. Please suspend the process."));
+			return;
+		}
+
 		if(inUse) {
 			QMessageBox::StandardButton button = QMessageBox::question(nullptr, tr("Breakpoint Already In Use"), tr("This breakpoint is already being used. Do you want to replace it?"), QMessageBox::Yes | QMessageBox::Cancel);
 			if(button != QMessageBox::Yes) {
@@ -456,7 +478,7 @@ void HardwareBreakpoints::setReadWriteBP(int index, bool inUse, edb::address_t a
 			}
 		}
 
-		for(IThread::pointer thread : process->threads()) {
+		for(std::shared_ptr<IThread> &thread : process->threads()) {
 			State state;
 			thread->get_state(&state);
 
