@@ -18,11 +18,22 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "DialogHeap.h"
 #include "Configuration.h"
-#include "edb.h"
 #include "IDebugger.h"
+#include "IProcess.h"
 #include "ISymbolManager.h"
 #include "MemoryRegions.h"
+#include "Module.h"
+#include "Symbol.h"
+#include "IRegion.h"
 #include "Util.h"
+#include "edb.h"
+
+#ifdef ENABLE_GRAPH
+#include "GraphWidget.h"
+#include "GraphNode.h"
+#include "GraphEdge.h"
+#endif
+
 #include <QFileInfo>
 #include <QHeaderView>
 #include <QMessageBox>
@@ -34,12 +45,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <algorithm>
 #include <functional>
 
-#ifdef ENABLE_GRAPH
-#include "GraphWidget.h"
-#include "GraphNode.h"
-#include "GraphEdge.h"
-#endif
-
 #if QT_VERSION >= 0x050000
 #include <QtConcurrent>
 #elif QT_VERSION >= 0x040800
@@ -48,7 +53,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "ui_DialogHeap.h"
 
-namespace HeapAnalyzer {
+namespace HeapAnalyzerPlugin {
 
 #define PREV_INUSE     0x1
 #define IS_MMAPPED     0x2
@@ -154,12 +159,12 @@ void DialogHeap::on_tableView_doubleClicked(const QModelIndex &index) {
 // Desc:
 //------------------------------------------------------------------------------
 void DialogHeap::get_library_names(QString *libcName, QString *ldName) const {
-	
+
 	Q_ASSERT(libcName);
 	Q_ASSERT(ldName);
-	
+
 	if(edb::v1::debugger_core) {
-		if(IProcess *process = edb::v1::debugger_core->process()) {	
+		if(IProcess *process = edb::v1::debugger_core->process()) {
 			const QList<Module> libs = process->loaded_modules();
 
 			for(const Module &module: libs) {
@@ -200,7 +205,7 @@ void DialogHeap::get_library_names(QString *libcName, QString *ldName) const {
 // Desc:
 //------------------------------------------------------------------------------
 void DialogHeap::process_potential_pointer(const QHash<edb::address_t, edb::address_t> &targets, Result &result) {
-	
+
 	if(IProcess *process = edb::v1::debugger_core->process()) {
 		if(result.data.isEmpty()) {
 			edb::address_t pointer(0);
@@ -398,14 +403,14 @@ edb::address_t DialogHeap::find_heap_start_heuristic(edb::address_t end_address,
 	edb::address_t test_addr(0);
 	if(IProcess *process = edb::v1::debugger_core->process()) {
 		process->read_bytes(heap_symbol, &test_addr, edb::v1::pointer_size());
-	
+
 		if(test_addr != edb::v1::debugger_core->page_size()) {
 			return 0;
 		}
-	
+
 		return start_address;
 	}
-	
+
 	return 0;
 }
 
@@ -427,15 +432,14 @@ void DialogHeap::do_find() {
 
 		get_library_names(&libcName, &ldName);
 	#if defined(Q_OS_LINUX) || defined(Q_OS_FREEBSD) || defined(Q_OS_OPENBSD)
-		Symbol::pointer s = edb::v1::symbol_manager().find(libcName + "::__curbrk");
-		if(s) {
+
+		if(std::shared_ptr<Symbol> s = edb::v1::symbol_manager().find(libcName + "::__curbrk")) {
 			end_address = s->address;
 		} else {
-			qDebug() << "[Heap Analyzer] __curbrk symbol not found in libc, falling back on heuristic! This may or may not work.";	
+			qDebug() << "[Heap Analyzer] __curbrk symbol not found in libc, falling back on heuristic! This may or may not work.";
 		}
 
-		s = edb::v1::symbol_manager().find(ldName + "::__curbrk");
-		if(s) {
+		if(std::shared_ptr<Symbol> s = edb::v1::symbol_manager().find(ldName + "::__curbrk")) {
 			start_address = s->address;
 		} else {
 
@@ -455,14 +459,14 @@ void DialogHeap::do_find() {
 
 			// read the contents of those symbols
 			process->read_bytes(end_address, &end_address, edb::v1::pointer_size());
-			process->read_bytes(start_address, &start_address, edb::v1::pointer_size());	
+			process->read_bytes(start_address, &start_address, edb::v1::pointer_size());
 		}
 
 		// just assume it's the bounds of the [heap] memory region for now
 		if(start_address == 0 || end_address == 0) {
 
-			const QList<IRegion::pointer> &regions = edb::v1::memory_regions().regions();
-			for(IRegion::pointer region: regions) {
+			const QList<std::shared_ptr<IRegion>> &regions = edb::v1::memory_regions().regions();
+			for(const std::shared_ptr<IRegion> &region: regions) {
 
 				if(region->name() == "[heap]") {
 
@@ -485,7 +489,7 @@ void DialogHeap::do_find() {
 		if(start_address == 0 || end_address == 0) {
 			QMessageBox::critical(this, tr("Could not calculate heap bounds"), tr("Failed to calculate the bounds of the heap."));
 			return;
-		}	
+		}
 
 	#else
 		#error "Unsupported Platform"
@@ -551,7 +555,7 @@ void DialogHeap::on_btnGraph_clicked() {
 
 		while(!result_stack.isEmpty()) {
 			const Result *const result = result_stack.pop();
-			
+
 			GraphNode *node;
 			if(result->type == tr("Busy")) {
 				node = new GraphNode(graph, edb::v1::format_pointer(result->block), Qt::lightGray);
@@ -580,7 +584,7 @@ void DialogHeap::on_btnGraph_clicked() {
 		}
 #endif
 
-		for(const Result *result: result_map) {
+		Q_FOREACH(const Result *result, result_map) {
 			const edb::address_t addr = result->block;
 			if(nodes.contains(addr)) {
 				for(edb::address_t pointer: result->points_to) {
