@@ -17,13 +17,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "DialogProcessProperties.h"
-#include "edb.h"
-#include "IDebugger.h"
-#include "MemoryRegions.h"
 #include "Configuration.h"
-#include "ISymbolManager.h"
 #include "DialogStrings.h"
+#include "IDebugger.h"
+#include "IProcess.h"
+#include "IRegion.h"
+#include "ISymbolManager.h"
+#include "MemoryRegions.h"
+#include "Module.h"
 #include "Symbol.h"
+#include "edb.h"
 
 #include <QDateTime>
 #include <QDebug>
@@ -42,7 +45,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "ui_DialogProcessProperties.h"
 
-namespace ProcessProperties {
+namespace ProcessPropertiesPlugin {
 
 namespace {
 
@@ -51,7 +54,7 @@ QString size_to_string(size_t n) {
 	static constexpr size_t KiB = 1024;
 	static constexpr size_t MiB = KiB * 1024;
 	static constexpr size_t GiB = MiB * 1024;
-	
+
 	if(n < KiB) {
 		return QString::number(n);
 	} else if(n < MiB) {
@@ -291,10 +294,10 @@ DialogProcessProperties::DialogProcessProperties(QWidget *parent) : QDialog(pare
 
 	threads_model_ = new ThreadsModel(this);
 	threads_filter_ = new QSortFilterProxyModel(this);
-	
+
 	threads_filter_->setSourceModel(threads_model_);
 	threads_filter_->setFilterCaseSensitivity(Qt::CaseInsensitive);
-	
+
 	ui->threadTable->setModel(threads_filter_);
 }
 
@@ -315,12 +318,13 @@ void DialogProcessProperties::updateGeneralPage() {
 	    if(IProcess *process = edb::v1::debugger_core->process()) {
 	        const QString exe            = process->executable();
 	        const QString cwd            = process->current_working_directory();
-			
+
 			std::shared_ptr<IProcess> parent = process->parent();
 	        const edb::pid_t parent_pid  = parent ? parent->pid() : 0;
 	        const QString parent_exe     = parent ? parent->executable() : QString();
-			
+
 	        const QList<QByteArray> args = process->arguments();
+			Q_UNUSED(args);
 
 			ui->editImage->setText(exe);
 			ui->editCommand->setText(QString());
@@ -376,11 +380,10 @@ void DialogProcessProperties::updateMemoryPage() {
 
 	if(edb::v1::debugger_core) {
 		edb::v1::memory_regions().sync();
-		const QList<IRegion::pointer> regions = edb::v1::memory_regions().regions();
+		const QList<std::shared_ptr<IRegion>> regions = edb::v1::memory_regions().regions();
 		ui->tableMemory->setSortingEnabled(false);
 
-
-		for(const IRegion::pointer &r: regions) {
+		for(const std::shared_ptr<IRegion> &r: regions) {
 			const int row = ui->tableMemory->rowCount();
 			ui->tableMemory->insertRow(row);
 			ui->tableMemory->setItem(row, 0, new QTableWidgetItem(edb::v1::format_pointer(r->start()))); // address
@@ -428,7 +431,7 @@ void DialogProcessProperties::updateEnvironmentPage(const QString &filter) {
 				const QString env_name  = env.mid(0, env.indexOf("="));
 				const QString env_value = env.mid(env.indexOf("=") + 1);
 
-				if(lower_filter.isEmpty() || env_name.toLower().contains(lower_filter)) {
+				if(lower_filter.isEmpty() || env_name.contains(lower_filter, Qt::CaseInsensitive)) {
 					const int row = ui->tableEnvironment->rowCount();
 					ui->tableEnvironment->insertRow(row);
 					ui->tableEnvironment->setItem(row, 0, new QTableWidgetItem(env_name));
@@ -513,13 +516,13 @@ void DialogProcessProperties::on_btnParent_clicked() {
 
 	if(edb::v1::debugger_core) {
 		if(IProcess *process = edb::v1::debugger_core->process()) {
-		
+
 			std::shared_ptr<IProcess> parent = process->parent();
-	        const QString parent_exe     = parent ? parent->executable() : QString();					
+	        const QString parent_exe = parent ? parent->executable() : QString();
 
 			QFileInfo info(parent_exe);
 			QDir dir = info.absoluteDir();
-			QDesktopServices::openUrl(QUrl(tr("file:///%1").arg(dir.absolutePath()), QUrl::TolerantMode));
+			QDesktopServices::openUrl(QUrl(tr("file://%1").arg(dir.absolutePath()), QUrl::TolerantMode));
 		}
 	}
 }
@@ -532,7 +535,7 @@ void DialogProcessProperties::on_btnImage_clicked() {
 	if(edb::v1::debugger_core) {
 		QFileInfo info(ui->editImage->text());
 		QDir dir = info.absoluteDir();
-		QDesktopServices::openUrl(QUrl(tr("file:///%1").arg(dir.absolutePath()), QUrl::TolerantMode));
+		QDesktopServices::openUrl(QUrl(tr("file://%1").arg(dir.absolutePath()), QUrl::TolerantMode));
 	}
 }
 
@@ -586,16 +589,16 @@ void DialogProcessProperties::updateThreads() {
 	threads_model_->clear();
 
 	if(IProcess *process = edb::v1::debugger_core->process()) {
-	
-		IThread::pointer current = process->current_thread();
-	
-		for(IThread::pointer thread : process->threads()) {
+
+		std::shared_ptr<IThread> current = process->current_thread();
+
+		for(std::shared_ptr<IThread> &thread : process->threads()) {
 
 			if(thread == current) {
 				threads_model_->addThread(thread, true);
 			} else {
 				threads_model_->addThread(thread, false);
-			}		
+			}
 		}
 	}
 }
