@@ -25,7 +25,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 class QListWidgetItem;
 
-namespace References {
+namespace ReferencesPlugin {
 
 namespace Ui { class DialogReferences; }
 
