@@ -19,26 +19,25 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef DATAVIEWINFO_20100101_H_
 #define DATAVIEWINFO_20100101_H_
 
-
-#include "RegionBuffer.h"
-#include "IRegion.h"
 #include <memory>
 
+#include <QtGlobal>
+
 class QHexView;
+class RegionBuffer;
+class IRegion;
 
 class DataViewInfo {
 public:
-	typedef std::shared_ptr<DataViewInfo> pointer;
-public:
-	explicit DataViewInfo(const IRegion::pointer &r);
+	explicit DataViewInfo(const std::shared_ptr<IRegion> &r);
 	~DataViewInfo();
 
 private:
 	Q_DISABLE_COPY(DataViewInfo)
 
 public:
-	IRegion::pointer         region;
-	RegionBuffer *const      stream;
+	std::shared_ptr<IRegion>  region;
+	RegionBuffer *const       stream;
 	std::shared_ptr<QHexView> view;
 
 public:
