@@ -19,17 +19,22 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef REGION_BUFFER_20101111_H_
 #define REGION_BUFFER_20101111_H_
 
-#include <QIODevice>
 #include "IRegion.h"
 
+#include <QIODevice>
+
+#include <memory>
+
+class IRegion;
+
 class RegionBuffer : public QIODevice {
 	Q_OBJECT
 public:
-	RegionBuffer(const IRegion::pointer &region);
-	RegionBuffer(const IRegion::pointer &region, QObject *parent);
+	RegionBuffer(const std::shared_ptr<IRegion> &region);
+	RegionBuffer(const std::shared_ptr<IRegion> &region, QObject *parent);
 
 public:
-	void set_region(const IRegion::pointer &region);
+	void set_region(const std::shared_ptr<IRegion> &region);
 
 public:
 	virtual qint64 readData(char * data, qint64 maxSize);
@@ -38,7 +43,7 @@ public:
 	virtual bool isSequential() const { return false; }
 
 private:
-	IRegion::pointer region_;
+	std::shared_ptr<IRegion> region_;
 };
 
 #endif
