@@ -1,7 +1,27 @@
+/*
+Copyright (C) 2017 - 2017 Evan Teran
+                          evan.teran@gmail.com
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 #ifndef ERRNO_NAMES_LINUX_H_20160514
 #define ERRNO_NAMES_LINUX_H_20160514
+
 namespace {
-const char* errnoNames[]={
+
+const char* errnoNames[] = {
 0,                       /*  0  */
 "EPERM",                 /*  1  */
 "ENOENT",                /*  2  */
@@ -535,4 +555,5 @@ const char* errnoNames[]={
 "EIOCBRETRY",            /* 530 */
 };
 }
+
 #endif
