@@ -20,44 +20,50 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "ArchProcessor.h"
 #include "BinaryString.h"
 #include "Configuration.h"
+#include "DebugEventHandlers.h"
 #include "Debugger.h"
 #include "DialogInputBinaryString.h"
 #include "DialogInputValue.h"
 #include "DialogOptions.h"
-#include "Debugger.h"
 #include "Expression.h"
-#include "Prototype.h"
+#include "ExpressionDialog.h"
+#include "IBreakpoint.h"
 #include "IDebugger.h"
 #include "IPlugin.h"
-#include "MD5.h"
+#include "IProcess.h"
+#include "IRegion.h"
 #include "MemoryRegions.h"
+#include "Prototype.h"
 #include "QHexView"
 #include "State.h"
+#include "Symbol.h"
 #include "SymbolManager.h"
 #include "version.h"
 
 #include <QAction>
 #include <QAtomicPointer>
 #include <QByteArray>
+#include <QCompleter>
+#include <QCoreApplication>
 #include <QDomDocument>
 #include <QFile>
 #include <QFileInfo>
-#include <QInputDialog>
 #include <QMessageBox>
-#include <QLabel>
-#include <QCoreApplication>
+#include <QCryptographicHash>
 
+#include <QDebug>
 #include <cctype>
 
-IDebugger *edb::v1::debugger_core = 0;
-QWidget   *edb::v1::debugger_ui   = 0;
+IDebugger *edb::v1::debugger_core = nullptr;
+QWidget   *edb::v1::debugger_ui   = nullptr;
+
 
 namespace {
 
 	typedef QList<IBinary::create_func_ptr_t> BinaryInfoList;
 
-	QAtomicPointer<IDebugEventHandler> g_DebugEventHandler = 0;
-	QAtomicPointer<IAnalyzer>          g_Analyzer          = 0;
+	DebugEventHandlers                 g_DebugEventHandlers;
+    QAtomicPointer<IAnalyzer>          g_Analyzer          = nullptr;
 	QMap<QString, QObject *>           g_GeneralPlugins;
 	BinaryInfoList                     g_BinaryInfoList;
 	CapstoneEDB::Formatter             g_Formatter;
@@ -73,19 +79,21 @@ namespace {
 		Q_ASSERT(value);
 		Q_ASSERT(offset);
 
-		bool ret = false;
-		*offset = 0;
-		const Symbol::pointer s = edb::v1::symbol_manager().find_near_symbol(address);
-		if(s) {
+		if(const std::shared_ptr<Symbol> s = edb::v1::symbol_manager().find_near_symbol(address)) {
 			*value = s->name;
 			*offset = address - s->address;
-			ret = true;
+			return true;
 		}
-		return ret;
+		
+		*offset = 0;
+		return false;
 	}
 }
 
 namespace edb {
+
+const char version[] = "1.0.0";
+
 namespace internal {
 
 //------------------------------------------------------------------------------
@@ -117,8 +125,9 @@ void load_function_db() {
 
 
 				Prototype func;
-				func.name = function.attribute("name");
-				func.type = function.attribute("type");
+				func.name     = function.attribute("name");
+				func.type     = function.attribute("type");
+				func.noreturn = function.attribute("noreturn", "false") == "true";
 
 				QDomElement argument = function.firstChildElement("argument");
 				for (; !argument.isNull(); argument = argument.nextSiblingElement("argument")) {
@@ -190,7 +199,7 @@ address_t cpu_selected_address() {
 // Name: current_cpu_view_region
 // Desc:
 //------------------------------------------------------------------------------
-IRegion::pointer current_cpu_view_region() {
+std::shared_ptr<IRegion> current_cpu_view_region() {
 	return ui()->ui.cpuView->region();
 }
 
@@ -253,24 +262,27 @@ IAnalyzer *analyzer() {
 }
 
 //------------------------------------------------------------------------------
-// Name: set_debug_event_handler
+// Name: execute_debug_event_handlers
 // Desc:
 //------------------------------------------------------------------------------
-IDebugEventHandler *set_debug_event_handler(IDebugEventHandler *p) {
-	Q_ASSERT(p);
-	return g_DebugEventHandler.fetchAndStoreAcquire(p);
+EVENT_STATUS execute_debug_event_handlers(const std::shared_ptr<IDebugEvent> &e) {
+	return g_DebugEventHandlers.execute(e);
 }
 
 //------------------------------------------------------------------------------
-// Name: debug_event_handler
+// Name: add_debug_event_handler
 // Desc:
 //------------------------------------------------------------------------------
-IDebugEventHandler *debug_event_handler() {
-#if QT_VERSION >= 0x050000
-	return g_DebugEventHandler.load();
-#else
-	return g_DebugEventHandler;
-#endif
+void add_debug_event_handler(IDebugEventHandler *p) {
+	g_DebugEventHandlers.add(p);
+}
+
+//------------------------------------------------------------------------------
+// Name: remove_debug_event_handler
+// Desc:
+//------------------------------------------------------------------------------
+void remove_debug_event_handler(IDebugEventHandler *p) {
+	g_DebugEventHandlers.remove(p);
 }
 
 //------------------------------------------------------------------------------
@@ -344,8 +356,8 @@ bool dump_data(address_t address) {
 // Desc:
 //------------------------------------------------------------------------------
 void set_breakpoint_condition(address_t address, const QString &condition) {
-	IBreakpoint::pointer bp = find_breakpoint(address);
-	if(bp) {
+
+	if(std::shared_ptr<IBreakpoint> bp = find_breakpoint(address)) {
 		bp->condition = condition;
 	}
 }
@@ -356,8 +368,8 @@ void set_breakpoint_condition(address_t address, const QString &condition) {
 //------------------------------------------------------------------------------
 QString get_breakpoint_condition(address_t address) {
 	QString ret;
-	IBreakpoint::pointer bp = find_breakpoint(address);
-	if(bp) {
+
+	if(std::shared_ptr<IBreakpoint> bp = find_breakpoint(address)) {
 		ret = bp->condition;
 	}
 
@@ -369,12 +381,12 @@ QString get_breakpoint_condition(address_t address) {
 // Name: create_breakpoint
 // Desc: adds a breakpoint at a given address
 //------------------------------------------------------------------------------
-IBreakpoint::pointer create_breakpoint(address_t address) {
+std::shared_ptr<IBreakpoint> create_breakpoint(address_t address) {
 
-	IBreakpoint::pointer bp;
+	std::shared_ptr<IBreakpoint> bp;
 
 	memory_regions().sync();
-	if(IRegion::pointer region = memory_regions().find_region(address)) {
+	if(std::shared_ptr<IRegion> region = memory_regions().find_region(address)) {
 		int ret = QMessageBox::Yes;
 
 		if(!region->executable() && config().warn_on_no_exec_bp) {
@@ -406,6 +418,13 @@ IBreakpoint::pointer create_breakpoint(address_t address) {
 
 		if(ret == QMessageBox::Yes) {
 			bp = debugger_core->add_breakpoint(address);
+			if(!bp) {
+				QMessageBox::critical(
+						0,
+						QT_TRANSLATE_NOOP("edb", "Error Setting Breakpoint"),
+						QObject::tr("Failed to set breakpoint at address %1").arg(address.toPointerString()));
+				return bp;
+			}
 			repaint_cpu_view();
 		}
 
@@ -416,7 +435,7 @@ IBreakpoint::pointer create_breakpoint(address_t address) {
 			QT_TRANSLATE_NOOP("edb", "Error Setting Breakpoint"),
 			QT_TRANSLATE_NOOP("edb", "Sorry, but setting a breakpoint which is not in a valid region is not allowed."));
 	}
-	
+
 	return bp;
 }
 
@@ -426,7 +445,7 @@ IBreakpoint::pointer create_breakpoint(address_t address) {
 //------------------------------------------------------------------------------
 address_t enable_breakpoint(address_t address) {
 	if(address != 0) {
-		IBreakpoint::pointer bp = find_breakpoint(address);
+		std::shared_ptr<IBreakpoint> bp = find_breakpoint(address);
 		if(bp && bp->enable()) {
 			return address;
 		}
@@ -440,7 +459,7 @@ address_t enable_breakpoint(address_t address) {
 //------------------------------------------------------------------------------
 address_t disable_breakpoint(address_t address) {
 	if(address != 0) {
-		IBreakpoint::pointer bp = find_breakpoint(address);
+		std::shared_ptr<IBreakpoint> bp = find_breakpoint(address);
 		if(bp && bp->disable()) {
 			return address;
 		}
@@ -496,13 +515,16 @@ bool eval_expression(const QString &expression, address_t *value) {
 // Desc:
 //------------------------------------------------------------------------------
 bool get_expression_from_user(const QString &title, const QString prompt, address_t *value) {
-	bool ok;
-    const QString text = QInputDialog::getText(debugger_ui, title, prompt, QLineEdit::Normal, QString(), &ok);
+    bool retval = false;
+    ExpressionDialog *inputDialog = new ExpressionDialog(title, prompt);
 
-	if(ok && !text.isEmpty()) {
-		return eval_expression(text, value);
-	}
-	return false;
+    if(inputDialog->exec())
+    {
+        *value = inputDialog->getAddress();
+        retval = true;
+    }
+    delete inputDialog;
+    return retval;
 }
 
 //------------------------------------------------------------------------------
@@ -577,6 +599,38 @@ Configuration &config() {
 	return g_Configuration;
 }
 
+//------------------------------------------------------------------------------
+// Name: get_human_string_at_address
+// Desc: attempts to create a summary of the content at address appropriate for
+// display in a user interface.
+// Note: strings are comprised of printable characters and whitespace.
+// Note: found_length is needed because we replace characters which need an
+//       escape char with the escape sequence (thus the resultant string may be
+//       longer than the original). found_length is the original length.
+//------------------------------------------------------------------------------
+
+bool get_human_string_at_address(address_t address, QString &s) {
+	bool ret = false;
+	if (address > 0x10000ULL) { // FIXME use page size
+		QString string_param;
+		int string_length;
+
+		if (get_ascii_string_at_address(address, string_param, edb::v1::config().min_string_length, 256, string_length)) {
+			ret = true;
+			s.append(
+				QString("ASCII \"%1\" ").arg(string_param)
+			);
+		} else if (get_utf16_string_at_address(address, string_param, edb::v1::config().min_string_length, 256, string_length)) {
+			ret = true;
+			s.append(
+				QString("UTF16 \"%1\" ").arg(string_param)
+			);
+		}
+	}
+	return ret;
+}
+
+
 //------------------------------------------------------------------------------
 // Name: get_ascii_string_at_address
 // Desc: attempts to get a string at a given address whose length is >= min_length
@@ -688,7 +742,13 @@ QString find_function_symbol(address_t address, const QString &default_value, in
 	int off;
 
 	if(function_symbol_base(address, &symname, &off)) {
-		symname = QString("%1+%2").arg(symname).arg(off, 0, 16);
+
+        if(config().function_offsets_in_hex) {
+            symname = QString("%1+0x%2").arg(symname).arg(off, 0, 16);
+        } else {
+            symname = QString("%1+%2").arg(symname).arg(off, 0, 10);
+        }
+
 		if(offset) {
 			*offset = off;
 		}
@@ -728,8 +788,7 @@ address_t get_variable(const QString &s, bool *ok, ExpressionError *err) {
 	const Register reg = state.value(s);
 	*ok = reg.valid();
 	if(!*ok) {
-		const Symbol::pointer sym = edb::v1::symbol_manager().find(s);
-		if(sym) {
+		if(const std::shared_ptr<Symbol> sym = edb::v1::symbol_manager().find(s)) {
 			*ok = true;
 			return sym->address;
 		}
@@ -764,7 +823,7 @@ address_t get_value(address_t address, bool *ok, ExpressionError *err) {
 	*ok = false;
 
 	if(IProcess *process = edb::v1::debugger_core->process()) {
-		*ok = process->read_bytes(address, &ret, edb::v1::pointer_size());
+		*ok = process->read_bytes(address, &ret, pointer_size());
 
 		if(!*ok) {
 			*err = ExpressionError(ExpressionError::CANNOT_READ_MEMORY);
@@ -776,7 +835,7 @@ address_t get_value(address_t address, bool *ok, ExpressionError *err) {
 
 //------------------------------------------------------------------------------
 // Name: get_instruction_bytes
-// Desc: attempts to read at most size bytes, but will retry using smaller sizes as needed
+// Desc: attempts to read at most size bytes.
 //------------------------------------------------------------------------------
 bool get_instruction_bytes(address_t address, quint8 *buf, int *size) {
 
@@ -785,13 +844,10 @@ bool get_instruction_bytes(address_t address, quint8 *buf, int *size) {
 	Q_ASSERT(*size >= 0);
 
 	if(IProcess *process = edb::v1::debugger_core->process()) {
-		bool ok = process->read_bytes(address, buf, *size);
-
-		while(!ok && *size) {
-			ok = process->read_bytes(address, buf, --(*size));
+		*size = process->read_bytes(address, buf, *size);
+		if (*size) {
+			return true;
 		}
-
-		return ok;
 	}
 
 	return false;
@@ -803,15 +859,27 @@ bool get_instruction_bytes(address_t address, quint8 *buf, int *size) {
 //       or NULL if none-found.
 // Note: the caller is responsible for deleting the object!
 //------------------------------------------------------------------------------
-std::unique_ptr<IBinary> get_binary_info(const IRegion::pointer &region) {
-	for(IBinary::create_func_ptr_t f: g_BinaryInfoList) {
-		std::unique_ptr<IBinary> p((*f)(region));
-
-		if(p->validate_header()) {
+std::unique_ptr<IBinary> get_binary_info(const std::shared_ptr<IRegion> &region) {
+	Q_FOREACH(IBinary::create_func_ptr_t f, g_BinaryInfoList) {
+		try {
+			std::unique_ptr<IBinary> p((*f)(region));
+			// reorder the list to put this successful plugin
+			// in front.
+			if (g_BinaryInfoList[0] != f) {
+				g_BinaryInfoList.removeOne(f);
+				g_BinaryInfoList.push_front(f);
+			}
 			return p;
+
+		} catch (std::exception e) {
+			// let's just ignore it...
 		}
 	}
 
+#if 0
+	qDebug() << "Failed to find any binary parser for region"
+		<< QString::number(region->start(), 16);
+#endif
 	return nullptr;
 }
 
@@ -826,7 +894,7 @@ address_t locate_main_function() {
 		if(IProcess *process = debugger_core->process()) {
 			const address_t address = process->code_address();
 			memory_regions().sync();
-			if(IRegion::pointer region = memory_regions().find_region(address)) {
+			if(std::shared_ptr<IRegion> region = memory_regions().find_region(address)) {
 				if(auto binfo = get_binary_info(region)) {
 					const address_t main_func = binfo->calculate_main();
 					if(main_func != 0) {
@@ -855,7 +923,7 @@ const QMap<QString, QObject *> &plugin_list() {
 // Desc: gets a pointer to a plugin based on it's classname
 //------------------------------------------------------------------------------
 IPlugin *find_plugin_by_name(const QString &name) {
-	for(QObject *p: g_GeneralPlugins) {
+	Q_FOREACH(QObject *p, g_GeneralPlugins) {
 		if(name == p->metaObject()->className()) {
 			return qobject_cast<IPlugin *>(p);
 		}
@@ -891,19 +959,20 @@ const Prototype *get_function_info(const QString &function) {
 // Note: make sure that memory regions has been sync'd first or you will likely
 //       get a null-region result
 //------------------------------------------------------------------------------
-IRegion::pointer primary_data_region() {
+std::shared_ptr<IRegion> primary_data_region() {
 
 	if(debugger_core) {
 		if(IProcess *process = debugger_core->process()) {
 			const address_t address = process->data_address();
 			memory_regions().sync();
-			if(IRegion::pointer region = memory_regions().find_region(address)) {
+			if(std::shared_ptr<IRegion> region = memory_regions().find_region(address)) {
 				return region;
 			}
 		}
 	}
 
-	return IRegion::pointer();
+	qDebug() << "primary data region not found!";
+	return nullptr;
 }
 
 //------------------------------------------------------------------------------
@@ -912,32 +981,31 @@ IRegion::pointer primary_data_region() {
 // Note: make sure that memory regions has been sync'd first or you will likely
 //       get a null-region result
 //------------------------------------------------------------------------------
-IRegion::pointer primary_code_region() {
+std::shared_ptr<IRegion> primary_code_region() {
 
 #if defined(Q_OS_LINUX)
 	if(debugger_core) {
 		if(IProcess *process = debugger_core->process()) {
 			const address_t address = process->code_address();
 			memory_regions().sync();
-			if(IRegion::pointer region = memory_regions().find_region(address)) {
+			if(std::shared_ptr<IRegion> region = memory_regions().find_region(address)) {
 				return region;
 			}
 		}
 	}
-
-	return IRegion::pointer();
 #else
-	const QString process_executable = debugger_core->process_exe(debugger_core->pid());
+	const QString process_executable = debugger_core->process()->name();
 
 	memory_regions().sync();
-	const QList<IRegion::pointer> r = memory_regions().regions();
-	for(const IRegion::pointer &region: r) {
+	const QList<std::shared_ptr<IRegion>> r = memory_regions().regions();
+	for(const std::shared_ptr<IRegion> &region: r) {
 		if(region->executable() && region->name() == process_executable) {
 			return region;
 		}
 	}
-	return IRegion::pointer();
 #endif
+	return nullptr;
+	qDebug() << "primary code region not found!";
 }
 
 //------------------------------------------------------------------------------
@@ -946,7 +1014,7 @@ IRegion::pointer primary_code_region() {
 //------------------------------------------------------------------------------
 void pop_value(State *state) {
 	Q_ASSERT(state);
-	state->adjust_stack(sizeof(reg_t));
+	state->adjust_stack(pointer_size());
 }
 
 //------------------------------------------------------------------------------
@@ -987,7 +1055,7 @@ quint32 edb_version() {
 bool overwrite_check(address_t address, unsigned int size) {
 	bool firstConflict = true;
 	for(address_t addr = address; addr != (address + size); ++addr) {
-		IBreakpoint::pointer bp = find_breakpoint(addr);
+		std::shared_ptr<IBreakpoint> bp = find_breakpoint(addr);
 
 		if(bp && bp->enabled()) {
 			if(firstConflict) {
@@ -1063,10 +1131,8 @@ QByteArray get_md5(const QVector<quint8> &bytes) {
 // Desc:
 //------------------------------------------------------------------------------
 QByteArray get_md5(const void *p, size_t n) {
-	MD5 md5(p, n);
-
-	// make a deep copy because MD5 is about to go out of scope
-	return QByteArray(reinterpret_cast<const char *>(md5.digest()), 16);
+	auto b = QByteArray::fromRawData(reinterpret_cast<const char *>(p), n);
+	return QCryptographicHash::hash(b, QCryptographicHash::Md5);
 }
 
 //------------------------------------------------------------------------------
@@ -1079,8 +1145,9 @@ QByteArray get_file_md5(const QString &s) {
 	file.open(QIODevice::ReadOnly);
 	if(file.isOpen()) {
 		if(file.size() != 0) {
-			const QByteArray file_bytes = file.readAll();
-			return get_md5(file_bytes.data(), file_bytes.size());
+			QCryptographicHash hasher(QCryptographicHash::Md5);
+			hasher.addData(file.readAll());
+			return hasher.result();
 		}
 	}
 
@@ -1192,16 +1259,41 @@ QStringList parse_command_line(const QString &cmdline) {
 Result<address_t> string_to_address(const QString &s) {
 	QString hex(s);
 	hex.replace("0x","");
-	
+
 	bool ok;
 	address_t r = edb::address_t::fromHexString(hex.left(2*sizeof(edb::address_t)), &ok);
 	if(ok) {
 		return edb::v1::make_result(r);
 	}
-	
+
 	return Result<address_t>(QLatin1String("Error converting string to address"), 0);
 }
 
+//------------------------------------------------------------------------------
+// Name: format_bytes
+// Desc:
+//------------------------------------------------------------------------------
+QString format_bytes(const uint8_t *buffer, size_t count) {
+	QString bytes;
+
+	if(count != 0) {
+		bytes.reserve(count * 4);
+
+		auto it = buffer;
+
+		char buf[4];
+		qsnprintf(buf, sizeof(buf), "%02x", *it++ & 0xff);
+		bytes += buf;
+
+		while(it != buffer + count) {
+			qsnprintf(buf, sizeof(buf), " %02x", *it++ & 0xff);
+			bytes += buf;
+		}
+	}
+
+	return bytes;
+}
+
 //------------------------------------------------------------------------------
 // Name: format_bytes
 // Desc:
@@ -1286,11 +1378,22 @@ void clear_status() {
 // Name: find_breakpoint
 // Desc:
 //------------------------------------------------------------------------------
-IBreakpoint::pointer find_breakpoint(address_t address) {
+std::shared_ptr<IBreakpoint> find_breakpoint(address_t address) {
 	if(debugger_core) {
 		return debugger_core->find_breakpoint(address);
 	}
-	return IBreakpoint::pointer();
+	return nullptr;
+}
+
+//------------------------------------------------------------------------------
+// Name: find_triggered_breakpoint
+// Desc:
+//------------------------------------------------------------------------------
+std::shared_ptr<IBreakpoint> find_triggered_breakpoint(address_t address) {
+	if(debugger_core) {
+		return debugger_core->find_triggered_breakpoint(address);
+	}
+	return nullptr;
 }
 
 //------------------------------------------------------------------------------
@@ -1378,7 +1481,7 @@ address_t selected_stack_address() {
 			return hexview->selectedBytesAddress();
 		}
 	}
-	
+
 	return static_cast<address_t>(-1);
 }
 
@@ -1392,7 +1495,7 @@ size_t selected_stack_size() {
 			return hexview->selectedBytesSize();
 		}
 	}
-	
+
 	return 0;
 }
 
@@ -1406,7 +1509,7 @@ address_t selected_data_address() {
 			return hexview->selectedBytesAddress();
 		}
 	}
-	
+
 	return static_cast<address_t>(-1);
 }
 
@@ -1420,7 +1523,7 @@ size_t selected_data_size() {
 			return hexview->selectedBytesSize();
 		}
 	}
-	
+
 	return 0;
 }
 
