@@ -24,9 +24,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 class GraphicsScene : public QGraphicsScene {
 	Q_OBJECT
 public:
-	GraphicsScene(QObject *parent = 0);
-	GraphicsScene(const QRectF &sceneRect, QObject *parent = 0);
-	GraphicsScene(qreal x, qreal y, qreal width, qreal height, QObject *parent = 0);
+    GraphicsScene(QObject *parent = nullptr);
+    GraphicsScene(const QRectF &sceneRect, QObject *parent = nullptr);
+    GraphicsScene(qreal x, qreal y, qreal width, qreal height, QObject *parent = nullptr);
 	virtual ~GraphicsScene();
 
 Q_SIGNALS:
